/**
 * \file gre/groupview.h
 * \brief GRE::GROUPVIEW class definitions
 *
 * \if NODOC
 * $Id: groupview.h_v 1.18 2013/02/15 20:43:53 mju Exp $
 *
 * $Log: groupview.h_v $
 * Revision 1.18  2013/02/15 20:43:53  mju
 * Fix dox hdr.
 *
 * Revision 1.17  2012/03/30 17:27:46  mju
 * Use stereoshift.
 *
 * Revision 1.15  2011/03/21 17:57:35  mju
 * Remove overlayTileIfRendered.
 *
 * Revision 1.13  2008/08/08 16:49:45  mju
 * Add overlayTilesIfRendered.
 *
 * Revision 1.10  2007/05/07 17:55:53  mju
 * Add invalidateStereo2D.
 *
 * Revision 1.9  2007/04/30 19:57:12  mju
 * Add computeStereoValues.
 *
 * Revision 1.8  2007/04/27 17:47:17  mju
 * Add values for stereo.
 *
 * Revision 1.7  2005/10/19 20:06:44  mju
 * Use gre namespace instead of legacy typedefs.
 *
 * Revision 1.6  2005/09/14 21:51:28  mju
 * Add rendertype_ActiveElems.
 *
 * Revision 1.5  2005/07/19 17:06:54  mju
 * Add rendertype_Matte.
 *
 * Revision 1.4  2005/04/29 17:49:21  mju
 * Support rendertarget.
 *
 * Revision 1.3  2005/04/22 17:35:39  mju
 * Use renderbuffer class.
 *
 * Revision 1.2  2005/03/22 22:00:13  mju
 * Add resetRendered.
 *
 * Revision 1.1  2005/02/25 20:30:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_GROUPVIEW_H
#define	INC_GRE_GROUPVIEW_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_MI32_RENDERBUFFER_H
	#include <mi32/renderbuffer.h>
#endif

#ifndef	INC_MI32_ZVIEW_H
	#include <mi32/zview.h>
#endif


namespace GRE {

//===================================================================================================================

//! Container for layer information retained for each view.
class GROUPVIEW {
	public:

		//! Rendering buffer types.
		enum RENDERTYPE {
			RENDERTYPE_Group =		0,
			RENDERTYPE_ActiveElems,
			RENDERTYPE_Matte,
			RENDERTYPE_COUNT
			};

		//! Constructor.
		GROUPVIEW (
			GRE::GROUP *group,
			GRE::VIEW *view
			);

		~GROUPVIEW ();

		//! Check if existing renderings are still valid.
		void CheckRendered ();

		//! Compute values for stereo rendering.
		ERRVALUE ComputeStereoValues ();

		//! Compute all transformations.
		ERRVALUE ComputeTrans ();

		//! Draw all 'active' elements to overlay buffer.
		ERRVALUE DrawActiveOverlay ();

		//! Get 3D viewpoint.
		const GRE::VIEWPOINT3D* Get3dViewPoint (
			) const { return (m_pViewPoint3D); }

		//! Get device rectangle used for rendering.
		//! This is a nominal rectangle based on layer extents and may not account for
		//! symbology and marginalia.  Returned rectangle may not be suitable for clipping
		//! during drawing, but will be suitable for selection of source object elements.
		const LRECT2D& GetDeviceRectUsed (
			) const { return (m_CurDevRectUsed); }

		//! Get associated GRE::GROUP.
		GRE::GROUP* GetGroup (
			) const { return (m_group); }

		//! Get rendering buffer for specified rendering type (const).
		const RENDERBUFFER& GetRenderBuffer (
			RENDERTYPE RenderType									//!< Rendering type
			) const { return (m_RenderBuffer[RenderType]); }

		//! Get rendering buffer for specified rendering type (non-const).
		RENDERBUFFER& GetRenderBuffer (
			RENDERTYPE RenderType									//!< Rendering type
			) { return (m_RenderBuffer[RenderType]); }

		//! Get rotation angle.
		double GetRotationAngle (
			) const { return (m_RotationAngle); }

		//! Get rotation cosine.
		double GetRotationCos (
			) const { return (m_RotationCos); }

		//! Get rotation sine.
		double GetRotationSin (
			) const { return (m_RotationSin); }

		//! Determine if first row(column) of device is for right eye in interlaced stereo rendering.
		bool GetStereoInterlaceStartRight (
			) const { return (m_StereoInterlaceStartRight); }

		const STEREOSHIFT& GetStereoShift () const
			{ return (m_ZView.GetStereoShift()); }

		//! Get associated GRE::VIEW.
		GRE::VIEW* GetView (
			) const { return (m_view); }

		//! Get const ZVIEW reference.
		const ZVIEW& GetZView () const
			{ return (m_ZView); }

		//! Invalidate stereo rendering status so recomputes.
		void InvalidateStereo2D (
			) { m_ZView.InvalidateStereoShift(); }

		//! Determine if drawing in 3D.
		bool Is3D (
			) const { return (m_Is3D); }

		//! Determine if group overlaps view device display area.
		bool IsGroupOverlappedDevice (
			) const { return (m_GroupOverlapsDevice); }

		//! Reset rendered status so redraws.
		void ResetRendered (
			GRE::RENDERTARGET target = GRE::RENDERTARGET_GroupAll
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		GRE::GROUP *m_group;
		GRE::VIEW *m_view;
		RENDERBUFFER m_RenderBuffer[RENDERTYPE_COUNT];
		GRE::VIEWPOINT3D *m_pViewPoint3D;				// 3D viewpoint
		DRECT2D m_NewTgtRectFull;						// New full rectangle in target device coordinates, based on transformation
		DRECT2D m_CurTgtRectFull;						// Current full rectangle in target device coordinates used by layer
		LRECT2D m_NewDevRectUsed;						// New clipped rectangle on device used, based on transformation
		LRECT2D m_CurDevRectUsed;						// Current clipped rectangle on device used
		double m_RotationAngle;							// Rotation angle
		double m_RotationCos;							// Rotation angle cosine
		double m_RotationSin;							// Rotation angle sine
		ZVIEW m_ZView;
		bool m_Is3D;										// Set if drawing in 3D
		bool m_GroupOverlapsDevice;
		bool m_StereoInterlaceStartRight;			// Set if first row/column of device is for right eye of interlaced stereo rendering

		GROUPVIEW (const GROUPVIEW&);
		GROUPVIEW& operator= (const GROUPVIEW&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_GROUPVIEW_H
