/**
 * \file mi32/coordtrans.h
 * \brief Coordinate Transformation (COORDTRANS) class definitions
 *
 * \if NODOC
 * $Id: coordtrans.h_v 1.7 2012/11/15 13:49:42 mju Exp $
 *
 * $Log: coordtrans.h_v $
 * Revision 1.7  2012/11/15 13:49:42  mju
 * Rename coordop_target as coordtrans_target.
 *
 * Revision 1.6  2012/11/09 21:11:11  mju
 * Export tps from dll.
 *
 * Revision 1.5  2012/11/09 18:04:01  mju
 * Move coordop_target defn here.
 *
 * Revision 1.4  2012/11/09 16:07:44  mju
 * Add coordtransi_identity defn.
 * Remove constructReverse (except base one).
 *
 * Revision 1.3  2012/11/08 18:29:53  mju
 * Don't nest coordtrans::target as will want to forward-declare later.
 *
 * Revision 1.2  2012/11/08 18:26:26  mju
 * Forward-declare coordop_target and add coordtrans::target.
 *
 * Revision 1.1  2012/11/08 17:46:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COORDTRANS_H
#define	INC_MI32_COORDTRANS_H

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_INTERLOCKEDINT32_H
#include <mi32/interlockedint32.h>
#endif

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

// Forward declarations.
#ifndef GENERATING_DOXYGEN_OUTPUT
struct CTRLPOINT3;
class THINPLATESPLINE;
namespace SPATREF {
	class COORDTRANS;
	}
#endif


namespace SPATREF {

//=====================================================================================================================

//! Target for densification operations.
class CLASSLIBEXPORT COORDTRANS_TARGET {
	public:

		//! Called before first input vertex processed.
		ERRVALUE OnBegin ()
			{ return (v_OnBegin()); }

		//! Called after last input vertex processed.
		ERRVALUE OnEnd ()
			{ return (v_OnEnd()); }

		//! Called for each dense point generated.
		ERRVALUE OnPoint (
			const double *pPoint,		//!< Pointer to point of output dimension
			INT32 InVertexNum,			//!< Vertex number in input array associated with point
			INT32 SubVertexNum			//!< Sub-vertex number, 0 if at input vertex
			) { return (v_OnPoint(pPoint,InVertexNum,SubVertexNum)); }

	private:
		// Overridables.

		//! Called before first input vertex processed.
		virtual ERRVALUE v_OnBegin ();

		//! Called after last input vertex processed.
		virtual ERRVALUE v_OnEnd ();

		//! Called for each dense point generated.
		virtual ERRVALUE v_OnPoint (
			const double *pPoint,		//!< Pointer to point of output dimension
			INT32 InVertexNum,			//!< Vertex number in input array associated with point
			INT32 SubVertexNum			//!< Sub-vertex number within densified 'segment', 0 if at input vertex
			) = 0;
	};

//=====================================================================================================================

//! Base for coordinate transformation implementation instance.
class CLASSLIBEXPORT COORDTRANSI {
	public:

		//! Instance implementation type.
		enum TYPE {
			TYPE_Undefined,		//!< Undefined, attempted use will result in error
			TYPE_Identity,			//!< Identity (COORDTRANSI_IDENTITY)
			TYPE_Linear,			//!< Linear (COORDTRANSI_LINEAR)
			TYPE_CheckRange,		//!< Check range of coordinates (COORDTRANSI_CHECKRANGE)
			TYPE_List,				//!< Sequence of transformations as a list (internally defined)
			TYPE_General,			//!< General single transformation
			};

		//! Common constructor.
		explicit COORDTRANSI (
			TYPE type = TYPE_General
			);

		//! Increment reference count.
		void AddRef ()
			{ ++m_RefCount; }

		//! Construct combined instance if possible.
		//! @return Newly constructed instance or NULL if cannot construct.
		COORDTRANSI* ConstructCombined (
			const COORDTRANSI *pNextInstance
			) const { return (v_ConstructCombined(*pNextInstance)); }

		//! Get reference count.
		INT32 GetRefCount () const
			{ return (m_RefCount); }

		//! Get transformation type.
		TYPE GetType () const
			{ return (m_Type); }

		//! Perform transformation on DPOINT2D array.
		int Process (
			DPOINT2D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity							//!< Validity array passed/returned, if NULL may return error if any point cannot be converted
			) const { return (v_Process2D(reinterpret_cast<double*>(points),NumPoints,validity)); }

		//! Perform transformation on DPOINT3D array.
		int Process (
			DPOINT3D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity							//!< Validity array passed/returned, if NULL may return error if any point cannot be converted
			) const { return (v_Process3D(reinterpret_cast<double*>(points),NumPoints,validity)); }

		//! Perform transformation on DPOINT3DH array.
		int Process (
			DPOINT3DH *points,						//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity							//!< Validity array passed/returned, if NULL may return error if any point cannot be converted
			) const { return (v_Process3DH(points,NumPoints,validity)); }

		//! Perform transformation 2D points in ND array.
		int Process2D (
			double *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity,							//!< Validity array passed/returned, if NULL may return error if any point cannot be converted
			int ndoubles								//!< Number of "double" values per point in array
			) const { return (v_Process2D(points,NumPoints,validity,ndoubles)); }

		//! Perform transformation 3D points in ND array.
		int Process3D (
			double *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity,							//!< Validity array passed/returned, if NULL may return error if any point cannot be converted
			int ndoubles								//!< Number of "double" values per point in array
			) const { return (v_Process3D(points,NumPoints,validity,ndoubles)); }

		//! Release reference, destroying if last one.
		void Release ();

	protected:

		//! Destructor.
		virtual ~COORDTRANSI ();

		//! Normalize points in DPOINT3DH array.
		static void Normalize3DH (DPOINT3DH *points, UINT32 NumPoints, UINT8 *validity);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MITHREAD::INTERLOCKEDINT32 m_RefCount;
		TYPE m_Type;

		// Not implemented.
		COORDTRANSI (const COORDTRANSI& rhs);
		COORDTRANSI& operator= (const COORDTRANSI& rhs);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Construct combined instance if possible.
		//! @return Pointer to combined instance or NULL if can't combine.
		//! Default implementation returns NULL.
		//! Simple combinations with "identity" or matching CRS are automatically handled by
		//! caller and do not need to be checked for.
		virtual COORDTRANSI* v_ConstructCombined (
			const COORDTRANSI& next
			) const;

		//! Construct reverse instance if possible.
		//! @return Pointer to reverse instance or NULL if can't construct.
		//! Default implementation returns NULL.
		virtual COORDTRANSI* v_ConstructReverse () const;

		//! Perform transformation on 2D points in ND array.
		//! Default implementation returns EProgramError, indicating transformation not supported.
		virtual int v_Process2D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd = 2) const;

		//! Perform transformation on 3D points in ND array.
		//! Default implementation calls Process2D.
		virtual int v_Process3D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd = 3) const;

		//! Perform transformation on DPOINT3DH array.
		//! Default implementation will "project" (normalize) points and call Process3D with nd = 4.
		virtual int v_Process3DH (DPOINT3DH *points, UINT32 NumPoints, UINT8 *validity) const;

	};

//=====================================================================================================================

//! Identity coordinate transformation.
class CLASSLIBEXPORT COORDTRANSI_IDENTITY : public COORDTRANSI {
	public:
		//! Get shared instance.
		//! Automatically increments reference count of returned pointer.
		static COORDTRANSI * GetShared ();
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		COORDTRANSI_IDENTITY ();
		virtual ~COORDTRANSI_IDENTITY ();
		// COORDTRANSI overrides.
		virtual int v_Process2D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		virtual int v_Process3D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		virtual int v_Process3DH (DPOINT3DH *points, UINT32 NumPoints, UINT8 *validity) const;
		#endif
	};

//=====================================================================================================================

//! Linear coordinate transformation.
class CLASSLIBEXPORT COORDTRANSI_LINEAR : public COORDTRANSI {
	public:

		//! Create transformation with MAT3X3.
		static void Create (
			COORDTRANS& trans,					//!< Transformation to contain result
			const MAT3X3& fwd,					//!< Forward transformation matrix
			const MAT3X3& rev,					//!< Reverse transformation matrix
			const COORDREFSYS& SourceCRS,		//!< Source CRS
			const COORDREFSYS& TargetCRS		//!< Target CRS
			);

		//! Create transformation with MAT4X4.
		static void Create (
			COORDTRANS& trans,					//!< Transformation to contain result
			const MAT4X4& fwd,					//!< Forward transformation matrix
			const MAT4X4& rev,					//!< Reverse transformation matrix
			const COORDREFSYS& SourceCRS,		//!< Source CRS
			const COORDREFSYS& TargetCRS		//!< Target CRS
			);

		//! Create scaling transformation.
		static void CreateScale (
			COORDTRANS& trans,					//!< Transformation to contain result
			const COORDREFSYS& SourceCRS,		//!< Source CRS
			const COORDREFSYS& TargetCRS,		//!< Target CRS
			double ScaleX,							//!< X scaling (forward direction)
			double ScaleY,							//!< Y scaling (forward direction)
			double ScaleZ = 1						//!< Z scaling (forward direction)
			);

		//! Construct from MAT3X3.
		explicit COORDTRANSI_LINEAR (
			const MAT3X3& mat3					//!< Forward transformation matrix
			);

		//! Construct from MAT4X4.
		explicit COORDTRANSI_LINEAR (
			const MAT4X4& mat4					//!< Forward transformation matrix
			);

		//! Construct from scale values.
		COORDTRANSI_LINEAR (
			double scaleX,							//!< X scaling
			double scaleY,							//!< Y scaling
			double scaleZ = 1						//!< Z scaling
			);

		//! Construct from scale and offset values.
		COORDTRANSI_LINEAR (
			const DPOINT3D& scale,				//!< Scaling
			const DPOINT3D& offset				//!< Offset (applied after scaling)
			);

		//! Determine if X, Y, Z are independent.
		//! True if output X depends only on input X, output Y on input Y and output Z on input Z.
		bool IsIndependent () const
			{ return (m_bIndependent); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MAT3X3 m_mat3;
		MAT4X4 m_mat4;
		bool m_bIndependent;
		bool m_bScaleOnly;
		bool m_bOffsetOnly;

		virtual ~COORDTRANSI_LINEAR ();

		// COORDTRANSI overrides.
		virtual COORDTRANSI* v_ConstructCombined (const COORDTRANSI& next) const;
		virtual int v_Process2D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		virtual int v_Process3D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		virtual int v_Process3DH (DPOINT3DH *points, UINT32 NumPoints, UINT8 *validity) const;
		#endif
	};

//=====================================================================================================================

//! Coordinate range check.
//! Provides ability to independently limit or check ranges for X, Y and Z.  Does not support DPOINT3DH processing.
class CLASSLIBEXPORT COORDTRANSI_CHECKRANGE : public COORDTRANSI {
	public:

		//! Handling mode for each ordinate.
		//! Not that if one of the combined modes is specified, the cyclic extents should be "larger" than the check/limit extents.
		//! If this is not the case, or if no separate cyclic extents are specified then no check/limit will be performed as it would
		//! have no additional effect.
		enum MODE {
			MODE_Ignore =					0,										//!< Ordinate ignored
			MODE_Check =					0x01,									//!< Check within CheckLimit range, setting as invalid if not
			MODE_Limit =					0x02,									//!< Limit values to CheckLimit range, adjusting if not
			MODE_Wrap =						0x04,									//!< Values "wrap" when outside cyclic range
			MODE_WrapAndCheck =			MODE_Wrap | MODE_Check,			//!< Wrap and then check (against narrower range)
			MODE_WrapAndLimit =			MODE_Wrap | MODE_Limit,			//!< Wrap and then limit (using narrower range)
			MODE_Oscillate =				0x08,									//!< Values "oscillate" between lower/upper part of cyclic range
			MODE_OscillateAndCheck =	MODE_Oscillate | MODE_Check,	//!< Oscillate and then check (against narrower range)
			MODE_OscillateAndLimit =	MODE_Oscillate | MODE_Limit,	//!< Oscillate and then limit (using narrower range)
			};

		//! Create transformation with X/Y extents.
		static void Create (
			COORDTRANS& trans,						//!< Transformation to contain result
			const COORDREFSYS& crs,					//!< CRS
			const DRECT2D& extent2d,				//!< Valid extents
			MODE modeX,									//!< X mode
			MODE modeY,									//!< Y mode
			const DRECT2D *pExtCyclic = 0			//!< Separate extents for cyclic modes, will use extent2d if needed and not specified
			);

		//! Create transformation with X/Y/Z extents.
		static void Create (
			COORDTRANS& trans,						//!< Transformation to contain result
			const COORDREFSYS& crs,					//!< CRS
			const DRECT3D& extent3d,				//!< Valid extents
			MODE modeX,									//!< X mode
			MODE modeY,									//!< Y mode
			MODE modeZ,									//!< Z mode
			const DRECT3D *pExtCyclic = 0			//!< Separate extents for cyclic modes, will use extent3d if needed and not specified
			);

		//! Construct with X/Y extents.
		COORDTRANSI_CHECKRANGE (
			const DRECT2D& extent2d,				//!< Valid extents
			MODE modeX,									//!< X mode
			MODE modeY,									//!< Y mode
			const DRECT2D *pExtCyclic = 0			//!< Separate extents for cyclic modes, will use extent2d if needed and not specified
			);

		//! Construct with X/Y/Z extents.
		COORDTRANSI_CHECKRANGE (
			const DRECT3D& extent3d,				//!< Valid extents
			MODE modeX,									//!< X mode
			MODE modeY,									//!< Y mode
			MODE modeZ,									//!< Z mode
			const DRECT3D *pExtCyclic = 0			//!< Separate extents for cyclic modes, will use extent3d if needed and not specified
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct CHECKER {
			DOUBLE_RANGE m_CheckLimit;
			DOUBLE_RANGE m_RangeCyclic;
			MODE m_mode;

			CHECKER ();
			void LimitOscillate (double& value) const;
			void LimitWrap (double& value) const;
			int Process (double *pOrdinate, UINT32 NumPoints, UINT8 *validity, int nd) const;
			void Setup (MODE mode, double CheckMin, double CheckMax, double WrapMin, double WrapMax);
			};

		CHECKER m_CheckerX;
		CHECKER m_CheckerY;
		CHECKER m_CheckerZ;

		COORDTRANSI_CHECKRANGE (const COORDTRANSI_CHECKRANGE& rhs);
		virtual ~COORDTRANSI_CHECKRANGE ();

		// COORDTRANSI overrides.
		virtual int v_Process2D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		virtual int v_Process3D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		#endif
	};

//=====================================================================================================================

//! Coordinate transformation using thin plate spline.
//! Only X and Y coordinates are transformed, Z coordinates will remain unchanged.
class CLASSLIBEXPORT COORDTRANSI_THINPLATESPLINE : public COORDTRANSI {
	public:

		//! Create transformation using CTRLPOINT3 array.
		static ERRVALUE Create (
			COORDTRANS& trans,					//!< Transformation to contain result
			const COORDREFSYS& SourceCRS,		//!< Source CRS
			const COORDREFSYS& TargetCRS,		//!< Target CRS
			const CTRLPOINT3 *CtrlPoints,		//!< Array of control points
			UINT32 NumPoints,						//!< Number of control points
			double tension							//!< Spline tension, 0 to fit all points exactly
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		THINPLATESPLINE *m_pTpsX;
		THINPLATESPLINE *m_pTpsY;

		COORDTRANSI_THINPLATESPLINE (THINPLATESPLINE *pTpsX, THINPLATESPLINE *pTpsY);
		virtual ~COORDTRANSI_THINPLATESPLINE ();

		// COORDTRANSI overrides.
		virtual int v_Process2D (double *pPoints, UINT32 NumPoints, UINT8 *validity, int nd) const;
		#endif
	};

//=====================================================================================================================

//! Bidirectional coordinate transformation.
class CLASSLIBEXPORT COORDTRANS {
	public:

		//! Default constructor, creates "undefined" transformation.
		COORDTRANS ();

		//! Copy constructor.
		//! Instances are reference-counted, so copying is efficient.
		COORDTRANS (
			const COORDTRANS& rhs
			);

		//! Destructor.
		~COORDTRANS ();

		//! Assignment.
		//! Instances are reference-counted, so assignment is efficient.
		COORDTRANS& operator= (
			const COORDTRANS& rhs
			);

		//! Add transformation to end of list.
		ERRVALUE AddEnd (
			const COORDTRANS& transformation,		//!< Operation to add
			bool CheckCRS									//!< If true, will check if current TargetCRS matches "transformation" SourceCRS and return error if not
			);

		//! Attempt to add transformations to end of list to achieve specified target CRS.
		//! @return TRUE if transformation(s) added, FALSE if not needed or error < 0
		int AddEnd (
			const COORDREFSYS& TargetCRS				//! Desired target coordinate reference system
			);

		//! Add transformation to start of list.
		ERRVALUE AddStart (
			const COORDTRANS& transformation,		//!< Operation to add
			bool CheckCRS									//!< If true, will check if current SourceCRS matches "transformation" TargetCRS and return error if not
			);

		//! Attempt to add transformations to start of list to achieve specified CRS.
		//! @return TRUE if transformation(s) added, FALSE if not needed or error < 0
		int AddStart (
			const COORDREFSYS& SourceCRS				//! Desired source coordinate reference system
			);

		//! Attach forward and reverse implementations with source/target CRS.
		void Attach (
			COORDTRANSI *pTransInstForward,			//!< Transformation from source to target, NULL if undefined
			COORDTRANSI *pTransInstReverse,			//!< Transformation from target to source, NULL if undefined
			const SPATREF::COORDREFSYS& SourceCRS,	//!< Source CRS
			const SPATREF::COORDREFSYS& TargetCRS,	//!< Target CRS
			bool DoAddRef									//!< Whether or not to increment reference count for each attached transformation
			);

		//! Clear transformation, reverting to undefined.
		void Clear ();

		//! Perform forward transformation on DPOINT2D array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertForward (
			DPOINT2D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array passed/returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceForward->Process(points,NumPoints,validity)); }

		//! Perform forward transformation on DPOINT3D array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertForward (
			DPOINT3D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceForward->Process(points,NumPoints,validity)); }

		//! Perform forward transformation on DPOINT3DH array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertForward (
			DPOINT3DH *points,						//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceForward->Process(points,NumPoints,validity)); }

		//! Perform forward transformation on DPOINT2D array with possible densification.
		ERRVALUE ConvertForwardDense (
			const DPOINT2D *ipoints,				//!< Input point array.
			UINT32 NumPoints,							//!< Number of points
			double tolerance,							//!< Densification tolerance in target coordinates
			COORDTRANS_TARGET& target				//!< Target for converted points
			) const;

		//! Perform forward transformation on DPOINT3D array with possible densification.
		ERRVALUE ConvertForwardDense (
			const DPOINT3D *ipoints,				//!< Input point array.
			UINT32 NumPoints,							//!< Number of points
			double tolerance,							//!< Densification tolerance in target coordinates
			COORDTRANS_TARGET& target				//!< Target for converted points
			) const;

		//! Perform reverse transformation on DPOINT2D array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertReverse (
			DPOINT2D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array passed/returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceReverse->Process(points,NumPoints,validity)); }

		//! Perform reverse transformation on DPOINT3D array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertReverse (
			DPOINT3D *points,							//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceReverse->Process(points,NumPoints,validity)); }

		//! Perform reverse transformation on DPOINT3DH array.
		//! @return TRUE (1) if all points successful, FALSE(0) if any point invalid, < 0 if error.
		//! If a "validity" array is specified it must initially contain nonzero values for every valid point.
		//! On return it will contain 0 for points that could not be completely transformed.
		//! If no validity array is specified, an error will be returned if any point cannot be converted.
		//! Any or all points in the array may be altered, regardless of return value.
		//! If all points fail to be converted an error will returned regardless of whether or not a validity array is specified.
		int ConvertReverse (
			DPOINT3DH *points,						//!< Point array passed/returned
			UINT32 NumPoints,							//!< Number of points to convert
			UINT8 *validity = 0						//!< Validity array returned, if NULL will return error if any point cannot be converted
			) const { return (m_pInstanceReverse->Process(points,NumPoints,validity)); }

		//! Perform reverse transformation on DPOINT2D array with possible densification.
		ERRVALUE ConvertReverseDense (
			const DPOINT2D *ipoints,				//!< Input point array.
			UINT32 NumPoints,							//!< Number of points
			double tolerance,							//!< Densification tolerance in target coordinates
			COORDTRANS_TARGET& target				//!< Target for converted points
			) const;

		//! Perform reverse transformation on DPOINT3D array with possible densification.
		ERRVALUE ConvertReverseDense (
			const DPOINT3D *ipoints,				//!< Input point array.
			UINT32 NumPoints,							//!< Number of points
			double tolerance,							//!< Densification tolerance in target coordinates
			COORDTRANS_TARGET& target				//!< Target for converted points
			) const;

		//! Get source coordinate reference system.
		const COORDREFSYS& GetSourceCRS () const 
			{ return (m_SourceCRS); }

		//! Get target coordinate reference system.
		const COORDREFSYS& GetTargetCRS () const 
			{ return (m_TargetCRS); }

		//! Determine if forward operation is supported.
		bool HasForward () const
			{ return (m_pInstanceForward->GetType() != COORDTRANSI::TYPE_Undefined); }

		//! Determine if reverse operation is supported.
		bool HasReverse () const
			{ return (m_pInstanceReverse->GetType() != COORDTRANSI::TYPE_Undefined); }

		//! Determine if both forward and reverse operations are supported.
		bool IsBidirectional () const
			{ return (HasForward() && HasReverse()); }

		//! Determine if transformation is defined.
		bool IsDefined () const
			{ return (HasForward() || HasReverse()); }

		//! Determine if is identity transformation in both directions.
		bool IsIdentity () const
			{ return (m_pInstanceForward->GetType() == COORDTRANSI::TYPE_Identity && m_pInstanceReverse->GetType() == COORDTRANSI::TYPE_Identity); }

		//! Reverse direction of transformation.
		void ReverseDirection ();

		//! Set as identity transformation.
		void SetIdentity (
			const SPATREF::COORDREFSYS& SourceCRS,	//!< Source CRS
			const SPATREF::COORDREFSYS& TargetCRS	//!< Target CRS
			);

		//! Set by automatically determining transformations from one CRS to another.
		//! Operation will be invalid if error occurs.
		ERRVALUE SetupAutoCRS (
			const COORDREFSYS& SourceCRS,		//!< Source coordinate reference system
			const COORDREFSYS& TargetCRS,		//!< Target coordinate reference system
			bool DisableRangeCheck = false	//!< Disable range check where supported
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		COORDTRANSI *m_pInstanceForward;
		COORDTRANSI *m_pInstanceReverse;
		COORDREFSYS m_SourceCRS;
		COORDREFSYS m_TargetCRS;

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace SPATREF

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_COORDTRANS_H
