/******************************************************************************
 *
 * \file mi32/mg2rast8.h
 * \brief A base raster-based MGD::DEVICE.  
 *
 * This is the 8-bit grayscale raster case.  There's an mg2rastc.h for
 * colormapped rasters which is derived from this one.
 *
 * This actually derives from MGD::DEVICE_MEM8G.  The raster reading and
 * writing is handled by overloading GetImagePointer() and a couple other
 * methods.
 *
 * \if NODOC
 * $Log: mg2rast8.h_v $
 * Revision 1.9  2011/10/26 17:58:14  mju
 * Frame buffer support.
 *
 * Revision 1.8  2006/06/27 18:03:21  scowan
 * Made objitem for create method non-const.
 *
 * Revision 1.7  2003/10/01 22:16:43  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2003/06/11 17:12:35  dwilliss
 * Virtual methods all renamed to start with v_ and made private (where
 * possible) or protected.
 *
 * Revision 1.4  2003/05/20 17:14:57  dwilliss
 * Virtual method wasn't actually virtual
 *
 * Revision 1.3  2003/05/19 17:48:30  dwilliss
 * Changed to use RVC classes instead of legacy stuff
 *
 * Revision 1.2  2003/05/19 16:27:09  dwilliss
 * Cleaned up a bit
 *
 * Revision 1.1  2003/05/19 15:22:37  dwilliss
 * Initial revision
 *
 *
 * \endif
 */
#ifndef INC_MI32_MG2RAST8_H
#define INC_MI32_MG2RAST8_H

#ifndef INC_MD2MEM8G_H
#include <mi32/mg2mem8g.h>
#endif

#ifndef INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

namespace MGD {


class CONTEXT_RASTER_GRAY8;

class DEVICE_RASTER_GRAY8: public DEVICE_MEM8G {

	public:

		//! Construct a 24-bit RGB raster device.
		//! After constructing, you should call Create()
		DEVICE_RASTER_GRAY8();

		//! Destructor
		virtual ~DEVICE_RASTER_GRAY8();

		// (Note: copy constructor private and unimplemented.  Can't copy one.)


		//! Create a raster to draw into.
		//! To draw into an existing raster, call Open
		ERRVALUE Create (
			RVC::OBJITEM& ObjItem,						//!< Information to create raster, location of created raster RETURNED
			RVC::RASTER::MAKEPARMS& MakeParms,
			const char* Source = 0
			);

		//! Open a raster to draw into.
		//! To draw into an existing raster, call Open
		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem
			);


		//! Write the pixel array back to the raster.
		//!
		//! If the raster is small (less than 1024x1024), we just allocate a
		//! buffer big enough for the whole raster.  In this case, nothing
		//! gets flushed until we call WritePixArray() (which the destructor
		//! will do if necessary)
		ERRVALUE WritePixArray (
			);


	protected:


		const RVC::RASTER& GetRaster (
			) const {
			return m_rast;
			}

		//! Write the pixel array from raster to buffer
		//!
		//! If the raster is small (less than 1024x1024), we just allocate a
		//! buffer big enough for the whole raster.  
		ERRVALUE ReadPixArray (
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class CONTEXT_RASTER_GRAY8;

		//! Copy constructor private and unimplemented
		DEVICE_RASTER_GRAY8(const DEVICE_RASTER_GRAY8&);

		int m_RealHeight;
		int m_RealWidth;
		bool m_bUseMemory;
		bool m_bDirty;			//!< need to flush
		RVC::RASTER m_rast;

		SIMPLE_ARRAY<UINT8> m_buffer;
		#endif // GENERATING_DOXYGEN_OUTPUT


		//! Create a GC for drawing.  
		//!
		//! Derived classes may override this and make it return anything
		//! derived from MGD_GC.  MGD_GC's constructor is private, so this
		//! is the only way for calling code to create a GC.
		virtual CONTEXT* v_CreateContext (
			);

		//! Called by WritePixArray just before it starts writing
		//! Colormapped derived class can use this to update colormap
		virtual ERRVALUE v_Flush (
			) {
			return 0;
			}

		//! In the Raster case, this will actually read the raster
		virtual UINT8* v_GetImagePointer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			);

		//! If GetImagePointer returns a pointer directly into the frame buffer,
		//!   this function does not need to be overridden.
		//! If GetImagePointer has to actually read something (say via an X
		//!   call to the display or reading out of a raster, then this version
		//!   should just return a temporary buffer big enough to hold a single
		//!   line.
		virtual UINT8* v_GetImagePointerNoRead (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			);
		
		//! Write an image buffer to the display
		//! If GetImagePointer reutrns a pointer directly into the frame buffer,
		//! this method can just return 0.  If GetImagePointer has to actually
		//! read something, this method is used to write the result back out.
		//! Typically used when drawing to a raster
		virtual ERRVALUE v_WriteImageBuffer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT8 *buf,
			UINT16 FrameBuffer
			);

		ERRVALUE Setup();
	};

class CONTEXT_RASTER_GRAY8 : public CONTEXT_MEM8G {
	public:
		// Note, default Constructor is protected

		virtual ~CONTEXT_RASTER_GRAY8();


	protected:

		//! Default Constructor 
		CONTEXT_RASTER_GRAY8 (
			DEVICE_RASTER_GRAY8* dev
			);

		DEVICE_RASTER_GRAY8* m_DevRast;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Copy constructor private and unimplemented
		CONTEXT_RASTER_GRAY8(const CONTEXT_RASTER_GRAY8&);

		friend class MGD::DEVICE_RASTER_GRAY8;	// Only DEVICE_RASTER_GRAY8::CreateContext() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};


} //! End of namespace MGD


#endif	// INC_MGDSIMPL_H

