/**
 * \file mi32/mimap3.h
 * \brief MIMAP3 template definition
 *
 * \if NODOC
 * $Id: mimap3.h_v 1.8 2011/06/20 20:02:29 mju Exp $
 *
 * $Log: mimap3.h_v $
 * Revision 1.8  2011/06/20 20:02:29  mju
 * Fix doc comments at beginning.
 *
 * Revision 1.7  2009/09/04 17:14:20  mju
 * Again try using typename to fix on mac.
 *
 * Revision 1.2  2009/09/02 17:18:41  mju
 * Don't use vector-bool as is slower than vector-uint8.
 *
 * Revision 1.1  2009/09/02 16:10:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MIMAP3_H
#define	INC_MI32_MIMAP3_H

#include <map>
#include <vector>

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


//! Template class allowing lookup of stored type by 'unsigned', 'double' or 'MISTRING'.
template <typename STORED_TYPE> class MIMAP3 {
	public:


		//! Constructor.
		//! If NumValues is nonzero, construction of 'NumValues' instances of STORED_TYPE will occur.
		//! Care should be taken to ensure that this value accurately reflects the true maximum value to be used
		//! and that the sparseness of the map is low.  Otherwise these values should be set to 0 to perform std::map based
		//! searching and insertion.
		explicit MIMAP3 (
			unsigned NumValues = 0,						//!< Initial number of values to allow if using 'unsigned' to access.
			unsigned MaxValues = 0
			):
			m_VectorStored(NumValues),
			m_VectorValid(NumValues,false),
			m_MaxValues((NumValues > MaxValues) ? NumValues : MaxValues),
			m_UseMap(m_MaxValues==0)
			{ }

		//! Find stored item using 'unsigned' value.
		bool Find (
			unsigned value,
			STORED_TYPE &ret
			) const {
				if (value >= m_VectorValid.size() || !m_VectorValid[value]) {
					if (m_UseMap) return (Find(static_cast<double>(value),ret));
					else return (false);
					}
				ret = m_VectorStored[value];
				return (true);
				}

		//! Find stored item using 'double' value.
		bool Find (
			double value,
			STORED_TYPE &ret
			) const {
				typename MAP_DOUBLE::const_iterator it = m_MapDouble.find(value);
				if (it == m_MapDouble.end()) return (false);
				ret = it->second;
				return (true);
				}

		//! Find stored item using 'MISTRING' value.
		bool Find (
			const MISTRING& value,
			STORED_TYPE &ret
			) const {
				typename MAP_STRING::const_iterator it = m_MapString.find(value);
				if (it == m_MapString.end()) return (false);
				ret = it->second;
				return (true);
				}

		//! Store item using 'unsigned' value.
		void Store (
			unsigned value,
			const STORED_TYPE& stored
			) {
			if (value >= m_MaxValues) {
				Store(static_cast<double>(value),stored);
				m_UseMap = true;
				}
			else {
				if (value >= m_VectorValid.size()) {
					m_VectorValid.resize(value+1,false);
					m_VectorStored.resize(value+1);
					}
				m_VectorValid[value] = true;
				m_VectorStored[value] = stored;
				}
			}

		//! Store item using 'double' value.
		void Store (
			double value,
			const STORED_TYPE& stored
			) { m_MapDouble.insert(std::pair<double,STORED_TYPE>(value,stored)); }

		//! Store item using 'MISTRING' value.
		void Store (
			const MISTRING& value,
			const STORED_TYPE& stored
			) { m_MapString.insert(std::pair<MISTRING,STORED_TYPE>(value,stored)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef std::map<double,STORED_TYPE> MAP_DOUBLE;
		typedef std::map<MISTRING,STORED_TYPE> MAP_STRING;

		std::vector<STORED_TYPE> m_VectorStored;
		std::vector<UINT8> m_VectorValid;
		MAP_DOUBLE m_MapDouble;
		MAP_STRING m_MapString;
		unsigned m_MaxValues;
		bool m_UseMap;

		#endif
	};

#endif	INC_MI32_MIMAP3_H
