/**
 * \file mi32/trans2dmapgen.h
 * \brief TRANS2D_MAPGEN definitions
 *
 * \if NODOC
 * $Id: trans2dmapgen.h_v 1.7 2012/11/09 18:35:35 mju Exp $
 *
 * $Log: trans2dmapgen.h_v $
 * Revision 1.7  2012/11/09 18:35:35  mju
 * Remove isReversible.
 *
 * Revision 1.6  2012/11/09 18:13:12  mju
 * Remove isGeneral.
 *
 * Revision 1.5  2012/11/09 16:10:27  mju
 * Rename method.
 *
 * Revision 1.4  2012/11/08 17:47:37  mju
 * Add setCoordTransFull.
 *
 * Revision 1.3  2012/06/12 14:46:32  mju
 * Use prevent_memfunc.
 *
 * Revision 1.2  2011/09/23 21:43:51  mju
 * Use trans2dmodel instead of uint16.
 *
 * Revision 1.1  2011/09/22 21:16:49  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_TRANS2DMAPGEN_H
#define	INC_MI32_TRANS2DMAPGEN_H

#ifndef	INC_MI32_TRANS2DMODEL_H
#include <mi32/trans2dmodel.h>
#endif

#ifndef	INC_MI32_CTRLPT_H
#include <mi32/ctrlpt.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

// Forward declarations
class MISTRING;
class PTTRIANGULATOR;
class TRANS2D_AFFINE;
namespace SPATREF {
	class COORDREFSYS;
	class COORDTRANS;
	}

//! Prototype for generic transformation function.
typedef int (*TRANSFUNC)(void*,void*,int,int,void*);
#endif

//=====================================================================================================================

//! General map transformation implementing Exterior - InputMap - OutputMap - Exterior sequence.
class CLASSLIBEXPORT TRANS2D_MAPGEN {
	public:

		//! Transformation direction.
		enum DIRECTION {
			DIRECTION_Forward = 0,		//!< Forward direction
			DIRECTION_Reverse = 1,		//!< Reverse direction
			DIRECTION_Inverse = DIRECTION_Reverse
			};

		enum SIDE {
			SIDE_Input = 0,
			SIDE_Output = 1
			};

		enum COORD {
			COORD_Exterior = 0,
			COORD_Map = 1
			};

		//! Default constructor, creates identity transformation.
		TRANS2D_MAPGEN ();

		//! Copy constructor.
		TRANS2D_MAPGEN (
			const TRANS2D_MAPGEN& rhs
			);

		//! Destructor.
		~TRANS2D_MAPGEN ();

		//! Assignment from TRANS2D_MAPGEN.
		TRANS2D_MAPGEN& operator= (
			const TRANS2D_MAPGEN& rhs
			);

		//! Apply offset from input to map.
		ERRVALUE ApplyOffsetInputToMap (
			double OffsetX,						//!< X offset in map coordinates
			double OffsetY							//!< Y offset in map coordinates
			);

		//! Apply offset from map to input.
		ERRVALUE ApplyOffsetMapToInput (
			double OffsetX,						//!< X offset in input coordinates
			double OffsetY							//!< Y offset in input coordinates
			);

		//! Apply offset from map to output.
		ERRVALUE ApplyOffsetMapToOutput (
			double OffsetX,						//!< X offset in output coordinates
			double OffsetY							//!< Y offset in output coordinates
			);

		//! Apply offset from output to map.
		ERRVALUE ApplyOffsetOutputToMap (
			double OffsetX,						//!< X offset in map coordinates
			double OffsetY							//!< Y offset in map coordinates
			);

		//! Apply scale from input to map.
		ERRVALUE ApplyScaleInputToMap (
			double ScaleX,							//!< X scaling
			double ScaleY							//!< Y scaling
			);

		//! Apply scale from map to input.
		ERRVALUE ApplyScaleMapToInput (
			double ScaleX,							//!< X scaling
			double ScaleY							//!< Y scaling
			);

		//! Apply scale from map to output.
		ERRVALUE ApplyScaleMapToOutput (
			double ScaleX,							//!< X scaling
			double ScaleY							//!< Y scaling
			);

		//! Apply scale from output to map.
		ERRVALUE ApplyScaleOutputToMap (
			double ScaleX,							//!< X scaling
			double ScaleY							//!< Y scaling
			);

		//! Clear transformation, resetting to identity.
		void Clear ();

		//! Perform forward transformation on DPOINT2D.
		ERRVALUE ConvertForward (
			const DPOINT2D& ipoint,				//!< Input point
			DPOINT2D& opoint						//!< Output point
			) const;

		//! Perform forward transformation on DPOINT3D.
		ERRVALUE ConvertForward (
			const DPOINT3D& ipoint,				//!< Input point
			DPOINT3D& opoint						//!< Output point
			) const;

		//! Perform forward transformation on DPOINT2D array.
		ERRVALUE ConvertForward (
			const DPOINT2D *ipoints,			//!< Input point array
			DPOINT2D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform forward transformation on DPOINT3D array.
		ERRVALUE ConvertForward (
			const DPOINT3D *ipoints,			//!< Input point array
			DPOINT3D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform forward transformation on generic point array.
		ERRVALUE ConvertForward (
			const void *ipoints,					//!< Input point array
			void *opoints,							//!< Output point array
			INT32 NumPoints,						//!< Number of points to convert
			int NumDim								//!< Number of dimensions
			) const;

		//! Perform forward transformation on rectangle.
		ERRVALUE ConvertForward (
			const DRECT2D& irect,				//!< Input rectangle
			DRECT2D& orect,						//!< Output rectangle
			int sidepoints=0						//!< Number of extra points along sides for interpolation
			) const;

		//! Perform forward transformation with densification.
		//! @return Number of output points or error < 0.
		INT32 ConvertForwardDense (
			const double *ipoints,				//!< Input point array
			INT32 iNumPoints,						//!< Number of input points
			int NumDim,								//!< Numer of dimensions
			MIDOUBLEARRAY& opoints				//!< Array of output points returned
			) const;

		//! Convert from "input" to "map" coordinates.
		ERRVALUE ConvertInputToMap (
			const DPOINT2D& ipoint,				//!< Point to convert
			DPOINT2D& opoint						//!< Point returned
			) const;

		//! Perform reverse transformation on DPOINT2D.
		ERRVALUE ConvertInverse (
			const DPOINT2D& ipoint,				//!< Input point
			DPOINT2D& opoint						//!< Output point
			) const;

		//! Perform reverse transformation on DPOINT3D.
		ERRVALUE ConvertInverse (
			const DPOINT3D& ipoint,				//!< Input point
			DPOINT3D& opoint						//!< Output point
			) const;

		//! Perform reverse transformation on DPOINT2D array.
		ERRVALUE ConvertInverse (
			const DPOINT2D *ipoints,			//!< Input point array
			DPOINT2D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform reverse transformation on DPOINT3D array.
		ERRVALUE ConvertInverse (
			const DPOINT3D *ipoints,			//!< Input point array
			DPOINT3D *opoints,					//!< Output point array
			INT32 NumPoints						//!< Number of points to convert
			) const;

		//! Perform reverse transformation on generic point array.
		ERRVALUE ConvertInverse (
			const void *ipoints,					//!< Input point array
			void *opoints,							//!< Output point array
			INT32 NumPoints,						//!< Number of points to convert
			int NumDim								//!< Number of dimensions
			) const;

		//! Perform reverse transformation on rectangle.
		ERRVALUE ConvertInverse (
			const DRECT2D& irect,				//!< Input rectangle
			DRECT2D& orect,						//!< Output rectangle
			int sidepoints=0						//!< Number of extra points along sides for interpolation
			) const;

		//! Perform reverse transformation with densification.
		//! @return Number of output points or error < 0.
		INT32 ConvertInverseDense (
			const double *ipoints,				//!< Input point array
			INT32 iNumPoints,						//!< Number of input points
			int NumDim,								//!< Numer of dimensions
			MIDOUBLEARRAY& opoints				//!< Array of output points returned
			) const;

		//! Convert from "map" to "input" coordinates.
		ERRVALUE ConvertMapToInput (
			const DPOINT2D& ipoint,				//!< Point to convert
			DPOINT2D& opoint						//!< Point returned
			) const;

		//! Convert from input map coordinates to output map coordinates.
		ERRVALUE ConvertMapToMapForward (
			const DPOINT2D& ipoint,				//!< Point to convert
			DPOINT2D& opoint						//!< Output point
			) const;

		//! Convert from output map coordinates to input map coordinates.
		ERRVALUE ConvertMapToMapInverse (
			const DPOINT2D& ipoint,				//!< Point to convert
			DPOINT2D& opoint						//!< Output point
			) const;

		//! Convert from "output" to "map" coordinates.
		ERRVALUE ConvertOutputToMap (
			const DPOINT2D& ipoint,				//!< Point to convert
			DPOINT2D& opoint						//!< Output point
			) const;

		//! Copy MAPGEN "input" parameters to "input" parameters.
		ERRVALUE CopyInputToInput (
			const TRANS2D_MAPGEN& itrans
			);

		//! Copy MAPGEN "input" parameters to "output" parameters.
		ERRVALUE CopyInputToOutput (
			const TRANS2D_MAPGEN& itrans
			);

		//! Copy MAPGEN "output" parameters to "input" parameters.
		ERRVALUE CopyOutputToInput (
			const TRANS2D_MAPGEN& itrans
			);

		//! Copy MAPGEN "output" parameters to "output" parameters.
		ERRVALUE CopyOutputToOutput (
			const TRANS2D_MAPGEN& itrans
			);

		//! Determine scale/rotation/shear at specified point for forward transformation.
		ERRVALUE DisectForward (
			const DPOINT2D& point,				//!< Reference point at which values will be computed
			double *xscale,						//!< X scale returned (0 if don't need)
			double *yscale,						//!< Y scale returned (0 if don't need)
			double *rotangle = 0,				//!< Rotation angle in radians returned (0 if don't need)
			double *shearangle = 0,				//!< Shear angle in radians returned (0 if don't need)
			double size = 1.0						//!< Size of area over which to compute values
			) const;

		//! Determine scale/rotation/shear at specified point for reverse transformation.
		ERRVALUE DisectInverse (
			const DPOINT2D& point,				//!< Reference point at which values will be computed
			double *xscale,						//!< X scale returned (0 if don't need)
			double *yscale,						//!< Y scale returned (0 if don't need)
			double *rotangle = 0,				//!< Rotation angle in radians returned (0 if don't need)
			double *shearangle = 0,				//!< Shear angle in radians returned (0 if don't need)
			double size = 1.0						//!< Size of area over which to compute values
			) const;

		//! Get affine transformation if available.
		//! Will return error if transformation is not affine.
		DEPRECATED ERRVALUE GetAffine (
			MAT3X3 fwd,								//!< Forward transformation
			MAT3X3 rev								//!< Reverse transformation
			) const;

		//! Get affine transformation if available.
		//! Will return error if transformation is not affine.
		ERRVALUE GetAffine (
			TRANS2D_AFFINE& trans				//!< Affine transformation
			) const;

		//! Get "input" control points.
		const DOUBLE_ARRAY<CTRLPOINT3>& GetInputCtrlPoints () const;

		//! Get "input" formula display.
		void GetInputFormulasDisplay (
			const MISTRING& ExtSymX,
			const MISTRING& ExtSymY,
			MISTRING& retstr						//!< Formulas returned, separated by newline, empty if not available
			) const;

		//! Get 'input' map coordinate reference system.
		const SPATREF::COORDREFSYS& GetInputMapCoordRefSys () const;

		//! Get input model
		TRANS2DMODEL GetInputModel () const;

		//! Get input transformation as affine.
		ERRVALUE GetInputTrans (
			TRANS2D_AFFINE& trans				//!< Affine transformation
			) const;

		//! Get input transformation as affine.
		DEPRECATED ERRVALUE GetInputTrans (
			MAT3X3 fwd,
			MAT3X3 rev
			) const;

		//! Get input transformation as control points.
		//! Will return number of points if any or error
		int GetInputTrans (
			CTRLPOINT3 **ctrlpoint				
			) const;

		//! Get local affine transformation.
		ERRVALUE GetLocalAffine (
			const DPOINT2D& point,				//!< Point at which to compute transformation
			TRANS2D_AFFINE& trans,				//!< Affine transformation returned
			double size = 1.0,					//!< Size for approximation
			SIDE side = SIDE_Input				//!< Side of transformation corresponding to specified point
			) const;

		//! Get local affine transformation.
		DEPRECATED ERRVALUE GetLocalAffine (
			const DPOINT2D& point,
			MAT3X3 fwd,								//!< Forward transformation
			MAT3X3 rev,								//!< Reverse transformation
			double size = 1.0,
			SIDE side = SIDE_Input
			) const;

		//! Get 2D node coordinates for piecewise transformation.
		bool GetNode2D (
			INT32 nodenum,
			DPOINT2D& point,
			SIDE side,
			COORD coord
			) const;

		//! Get 3D node coordinates for piecewise transformation.
		bool GetNode3D (
			INT32 nodenum,
			DPOINT3D& point,
			SIDE side,
			COORD coord
			) const;

		//! Get "output" control points.
		const DOUBLE_ARRAY<CTRLPOINT3>& GetOutputCtrlPoints () const;

		//! Get "output" formula display.
		void GetOutputFormulasDisplay (
			const MISTRING& ExtSymX,
			const MISTRING& ExtSymY,
			MISTRING& retstr						//!< Formulas returned, separated by newline, empty if not available
			) const;

		//! Get 'output' map coordinate reference system.
		const SPATREF::COORDREFSYS& GetOutputMapCoordRefSys () const;

		//! Get output model
		TRANS2DMODEL GetOutputModel () const;

		//! Get output transformation as affine.
		ERRVALUE GetOutputTrans (
			TRANS2D_AFFINE& trans				//!< Affine transformation
			) const;

		//! Get output transformation as affine.
		DEPRECATED ERRVALUE GetOutputTrans (
			MAT3X3 fwd,
			MAT3X3 rev
			) const;

		//! Get output transformation as control points.
		//! Will return number of points if any or error
		int GetOutputTrans (
			CTRLPOINT3 **ctrlpoint				
			) const;

		//! Get const TRIANGULATOR for input or output side.
		const PTTRIANGULATOR& GetTriangulator (
			SIDE side
			) const;

		//! Get non-const TRIANGULATOR for input or output side.
		//! Caller is responsible for updating piecewise transformation via UpdatePiecewise if changed.
		PTTRIANGULATOR& GetTriangulator (
			SIDE side
			);

		//! Determine if has forward transformation.
		bool HasForward () const;

		//! Determine if has reverse transformation.
		bool HasInverse () const;

		//! Determine if transformation is "affine".
		bool IsAffine () const;

		//! Determine if two transformations are "equal".
		bool IsEqual (
			const TRANS2D_MAPGEN& rhs
			) const;

		//! Determine if transformation is "identity".
		bool IsIdentity () const;

		//! Determine if 'input' transformation is 'identity'.
		bool IsInputTransIdentity () const;

		//! Determine if 'output' transformation is 'identity'.
		bool IsOutputTransIdentity () const;

		//! Reverse direction of transformation.
		void ReverseDirection ();

		//! Set active triangle for next point test.
		void SetActiveTriangle (
			INT32 triangle,
			SIDE side
			) const;

		//! Set full transformation to approximated affine if within specified tolerance.
		//! The approximate affine transformation will be retained until the next "Set" method is used
		//! at which time the transformation will revert to its "exact" mode (which may also be affine).
		void SetAffineApproximation (
			const DRECT2D& SrcRect,				//!< Rectangle in source coordinates
			double TgtTolerance,					//!< Tolerance relative target coordinates
			DIRECTION direction					//!< Direction from "source" to "target"
			);

		//! Set full transformation to approximated affine if within specified tolerance.
		//! The approximate affine transformation will be retained until the next "Set" method is used
		//! at which time the transformation will revert to its "exact" mode (which may also be affine).
		void SetAffineApproximation (
			const DRECT2D& SrcRect,				//!< Rectangle in source coordinates
			double TgtTolerance,					//!< Tolerance relative target coordinates
			SIDE side
			) { SetAffineApproximation(SrcRect,TgtTolerance,(side==SIDE_Input)?DIRECTION_Forward:DIRECTION_Reverse); }

		//! Set whether to exclude flipped pieces for piecewise transformation.
		//! If flipped pieces are excluded an error will be returned if an attempt is made to convert
		//! a point within a flipped piece.
		void SetExcludeFlippedPieces (
			bool ExcludeFlippedPieces
			);

		//! Set whether to perform extrapolation outside specified bounds.
		//! If extrapolation is not enabled then an exception -may- occur if and attempt is made
		//! to convert points outside the previously-specified input/output bounds.  This is the
		//! default condition and will generally be much faster than extrapolating.
		void SetExtrapolation (
			bool extrapolate						//!< Perform extrapolation if true, don't extrapolate if false
			);

		//! Set full transformation using COORDTRANS.
		//! While in effect, attempts to set other aspects of TRANS2D_MAPGEN or to use partial conversion will be
		//! ignored or result in an error.
		void SetFullTrans (
			const SPATREF::COORDTRANS& trans	//!< Coordinate transformation
			);

		//! Set entire transformation as affine.
		void SetFullTransAffine (
			const TRANS2D_AFFINE& trans		//!< Affine transformation
			);

		void SetGeneral (
			TRANSFUNC TransFuncFwd,
			TRANSFUNC TransFuncRev,
			void *userdata
			);

		//! Set boundaries for valid transformation in input coordinates
		ERRVALUE SetInputBounds (
			const DRECT2D& boundrect			//!< Bounding rectangle in input coordinates
			);

		//! Set maximum error for point densification in input coordinates.
		void SetInputMaxError (
			double maxerror						//!< Maximum error value to set
			);

		//! Set input map coordinate reference system.
		ERRVALUE SetInputMapCoordRefSys (
			const SPATREF::COORDREFSYS& MapCRS	//!< Map coordinate reference system
			);

		//! Set input transformation as affine.
		void SetInputTrans (
			const TRANS2D_AFFINE& trans		//!< Affine transformation
			);

		//! Set input transformation as affine.
		DEPRECATED void SetInputTrans (
			const MAT3X3 fwd,						//!< Forward transformation
			const MAT3X3 rev						//!< Reverse transformation
			);

		//! Set input transformation from control point array.
		ERRVALUE SetInputTrans (
			const CTRLPOINT3 *CtrlPoint,		//!< Control point array
			INT32 NumPoints,						//!< Number of control points
			TRANS2DMODEL Model					//!< Transformation model
			);

		//! Set input transformation to identity.
		void SetInputTransIdentity ();

		//! Set maximum number of vertices to insert if densifying points
		void SetMaxVertices (
			INT32 MaxVertices
			);

		//! Set boundaries for valid transformation in output coordinates.
		ERRVALUE SetOutputBounds (
			const DRECT2D& boundrect			//!< Bounding rectangle in output coordinates
			);

		//! Set maximum error for point densification in output coordinates.
		void SetOutputMaxError (
			double maxerror						//!< Maximum error value to set
			);

		//! Set output map coordinate reference system.
		ERRVALUE SetOutputMapCoordRefSys (
			const SPATREF::COORDREFSYS& MapCRS	//!< Map coordinate reference system
			);

		//! Set output transformation as affine.
		void SetOutputTrans (
			const TRANS2D_AFFINE& trans		//!< Affine transformation
			);

		//! Set output transformation as affine.
		DEPRECATED void SetOutputTrans (
			const MAT3X3 fwd,
			const MAT3X3 rev
			);

		//! Set output transformation from control point array.
		ERRVALUE SetOutputTrans (
			const CTRLPOINT3 *CtrlPoint,		//!< Control point array
			INT32 NumPoints,						//!< Number of control points
			TRANS2DMODEL Model					//!< Transformation model
			);

		//! Set output transformation to identity.
		void SetOutputTransIdentity ();

		//! Set primary transformation direction.
		//! The default transformation direction if not set explicitly is "forward".  This method
		//! is used in cases where the time to do a reverse transformation is much higher than in
		//! the primary direction.  This is commonly used for raster warping.
		void SetPrimaryDirection (
			DIRECTION direction					//!< DIRECTION_Forward or DIRECTION_Reverse
			);

		void Update ();

		ERRVALUE UpdatePiecewise (
			SIDE side
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

inline bool operator==(
	const TRANS2D_MAPGEN& lhs,
	const TRANS2D_MAPGEN& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!=(
	const TRANS2D_MAPGEN& lhs,
	const TRANS2D_MAPGEN& rhs
	) { return (!lhs.IsEqual(rhs)); }

#define	TRANSGENCOORD_Input		TRANS2D_MAPGEN::SIDE_Input
#define	TRANSGENCOORD_Output		TRANS2D_MAPGEN::SIDE_Output

#ifndef GENERATING_DOXYGEN_OUTPUT
PREVENT_MEMFUNC(TRANS2D_MAPGEN)
#endif

//=====================================================================================================================

#undef CLASSLIBEXPORT

#endif	// INC_MI32_TRANS2DMAPGEN_H
