/**
 *
 * \file ucregexp.h <mi32/ucregexp.h>
 * \brief Unicode Regular Expression parser
 *
 *	\if NODOC
 *	$Id: ucregexp.h_v 1.17 2006/06/14 14:40:29 mju Exp $
 *
 * $Log: ucregexp.h_v $
 * Revision 1.17  2006/06/14 14:40:29  mju
 * Remove ucstring hdr.
 *
 * Revision 1.16  2005/03/31 16:57:22  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.15  2004/09/03 15:15:24  scowan
 * Fixed comments so that it will not show up as 'mi32' in doxygen.
 *
 * Revision 1.14  2003/09/26 16:40:20  dwilliss
 * Gave a brief description for the class
 *
 * Revision 1.13  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.12  2003/07/09 15:17:09  dwilliss
 * Get rid of obsolete Mac 9 stuff
 *
 * Revision 1.11  2003/07/09 14:30:36  mju
 * Define CLASSLIBEXPORT.
 *
 * Revision 1.10  2002/07/05 14:31:51  dwilliss
 * Undefine USEEXPORTPRAGMA before the end of the include file
 *
 * Revision 1.9  2002/06/28 15:42:04  dwilliss
 * Don't advertise the non-const version of Exec except when compiling
 * misystem.dll.
 *
 * Revision 1.8  2002/06/28 15:39:26  dwilliss
 * Added const version of Exec method
 *
 * Revision 1.7  2000/10/16 16:32:20  dwilliss
 * Needed to specify "friend struct NAME", not "friend NAME"
 *
 * Revision 1.6  2000/06/13 16:58:40  dwilliss
 * Added an example to Compile() method
 *
 * Revision 1.5  2000/06/12 19:37:23  dwilliss
 * Maybe fix Genitor documentation comments.  It doesn't want to format
 * tables like the docs say it does.
 *
 * Revision 1.4  2000/06/12 16:44:18  dwilliss
 * Hopefully fixed documentation so Genitor won't mangle it.
 * Changed order of parameters to Exec(). flags last now, not first.
 *   (shouldn't be in use yet, so ok to change).
 *
 * Revision 1.3  2000/06/07 20:00:39  dwilliss
 * Renamed a #define for Mike
 *
 * Revision 1.2  2000/06/06 23:04:41  dwilliss
 * Added documentation
 *
 * Revision 1.1  2000/06/06 22:48:34  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_UCREGEXP_H
#define INC_MI32_UCREGEXP_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

/*
 * Error codes.
 */
#define _URE_OK               0
#define _URE_UNEXPECTED_EOS   -1
#define _URE_CCLASS_OPEN      -2
#define _URE_UNBALANCED_GROUP -3
#define _URE_INVALID_PROPERTY -4

/*
 * Options that can be combined for searching.
 */
#define URE_IGNORE_NONSPACING      0x00000001
#define URE_DOT_MATCHES_SEPARATORS 0x00000002

struct UREBUFFER;
struct URESYMTAB;
struct UREDSTATE;
struct URETRANS;



//! \class UCREGEXP
//! \brief Unicode Regular Expression parser
//!
//! <b>Assumptions:</b>
//!
//!  \li Regular expression and text already normalized.
//!  \li Conversion to lower case assumes a 1-1 mapping.
//!
//! A discussion of regular expressions is beyond the scope of this
//! document.  
//!
//! <b>Definitions:</b>
//!
//!  \li Separator - any one of U+2028, U+2029, '\\n', '\\r'.
//!
//! <b>Operators:</b>
//! <table>
//!  <tr><td>^</td><td>matches beginning of string.</td></tr>
//!  <tr><td>$</td><td>matches end of string.</td></tr>
//!  <tr><td>.</td><td>match any character.</td></tr>
//!  <tr><td>*</td><td>match zero or more of the last subexpression.</td></tr>
//!  <tr><td>+</td><td>match one or more of the last subexpression.</td></tr>
//!  <tr><td>?</td><td>match zero or one of the last subexpression.</td></tr>
//!  <tr><td>()</td><td>subexpression grouping.</td></tr>
//! </table>
//!
//!   <b>Notes:</b>
//!
//!      The "." operator normally does not match separators, but a flag is
//!      available for the Exec() method that will allow this operator to
//!      match a separator (URE_DOT_MATCHES_SEPARATORS).
//!
//! <b>Literals and Constants:</b>
//! <table>
//!  <tr><td>c</td><td>literal UCS2 character.}}
//!  <tr><td>\\x....</td><td>hexadecimal number of up to 4 digits.</td></tr>
//!  <tr><td>\\X....</td><td>hexadecimal number of up to 4 digits.</td></tr>
//!  <tr><td>\\u....</td><td>hexadecimal number of up to 4 digits.</td></tr>
//!  <tr><td>\\U....</td><td>hexadecimal number of up to 4 digits.</td></tr>
//!  </table>
//!
//! <b>Character classes:</b>
//!
//! <table>
//!  <tr><td>[...]</td>   <td> Character class.</td></tr>
//!  <tr><td>[^...]</td>   <td> Negated character class.</td></tr>
//!  <tr><td>\\pN1,N2,...,Nn</td>   <td> Character properties class.</td></tr>
//!  <tr><td>\\PN1,N2,...,Nn</td>   <td> Negated character properties class.</td></tr>
//! </table>
//!
//!  <b>POSIX character classes recognized:</b>
//!
//!    \li :alnum:
//!    \li :alpha:
//!    \li :cntrl:
//!    \li :digit:
//!    \li :graph:
//!    \li :lower:
//!    \li :print:
//!    \li :punct:
//!    \li :space:
//!    \li :upper:
//!    \li :xdigit:}
//!
//!   <b>Notes:</b>
//!
//!      Character property classes are \\p or \\P followed by a comma separated
//!      list of integers between 1 and 21.  These integers are references to
//!      the following character properties:
//!
//! <table>
//!        <tr><td>N  </td><td>Character Property</td></tr>
//!        <tr><td>-- </td><td> ------------------------</td></tr>
//!        <tr><td>1  </td><td>NONSPACING</td></tr>
//!        <tr><td>2  </td><td>COMBINING</td></tr>
//!        <tr><td>3  </td><td>NUMDIGIT</td></tr>
//!        <tr><td>4  </td><td>NUMOTHER</td></tr>
//!        <tr><td>5  </td><td>SPACESEP</td></tr>
//!        <tr><td>6  </td><td>LINESEP</td></tr>
//!        <tr><td>7  </td><td>PARASEP</td></tr>
//!        <tr><td>8  </td><td>CNTRL</td></tr>
//!        <tr><td>9  </td><td>PUA</td></tr>
//!        <tr><td>10 </td><td>UPPER</td></tr>
//!        <tr><td>11 </td><td>LOWER</td></tr>
//!        <tr><td>12 </td><td>TITLE</td></tr>
//!        <tr><td>13 </td><td>MODIFIER</td></tr>
//!        <tr><td>14 </td><td>OTHERLETTER</td></tr>
//!        <tr><td>15 </td><td>DASHPUNCT</td></tr>
//!        <tr><td>16 </td><td>OPENPUNCT</td></tr>
//!        <tr><td>17 </td><td>CLOSEPUNCT</td></tr>
//!        <tr><td>18 </td><td>OTHERPUNCT</td></tr>
//!        <tr><td>19 </td><td>MATHSYM</td></tr>
//!        <tr><td>20 </td><td>CURRENCYSYM</td></tr>
//!        <tr><td>21 </td><td>OTHERSYM</td></tr>
//! </table>
//!
class CLASSLIBEXPORT UCREGEXP {
public:

	UCREGEXP();

	~UCREGEXP();
	
	//! Compile a Regular Expression.
	//!
	//! A discussion of regular expressions is beyond the scope of this
	//! document.  For more information, log on to most any flavor of
	//! Unix (Linux, for example) and type "man 7 regex"
	//!
	//! <b>Example</b>
	//! The following example will form a regular expression which will
	//! search for one or more digits followed by either a "-" or "/"
	//! and one or more digits.
	//!
	//! @code
	//!		strtouc(expr, "[[:digit:]]+[-/][[:digit:]]+");
	//!		regexp.Compile(expr);
	//! @endcode
	//!
	//! @return 0 if expression compiled, negative value if not.
	//!    Note, return codes are not MicroImages error codes.
	int Compile (
		const MIUNICODE* expr,
		bool bIgnoreCase
		);

#ifdef MISYSTEMDLL
#ifndef GENERATING_DOXYGEN_OUTPUT
//!	This method should have been const.  It's still here so that we can
//!	export the non-const version from the DLL, but all newly compiled
//!	code will only see the const version.  Note:  This can go away after
//!	6.7 release
	int Exec (
		const MIUNICODE *text, 		//!< Text to search
		UINT32 *pMatchStart = 0,	//!< Offset into text of start of matching text (pass NULL if don't care)
		UINT32 *pMatchEnd = 0,		//!< Offset into text of end of matching text (pass NULL if don't care)
		UINT32 textlen = ~0,			//!< Default is to use ucstrlen(text)
		int flags = 0
		);
#endif //!< GENERATING_DOXYGEN_OUTPUT
#endif


	//! Execute a regular expression.
	//!
	//!	Returns true if a match was found, false if not.
	//! If a match was found, *pMatchStart and *pMatchEnd
	//! will be set to the offsets into the text where the
	//! matching text was.
	//!
	//! @param flags
	//!		URE_IGNORE_NONSPACING      
	//!		URE_DOT_MATCHES_SEPARATORS 
	//!
	bool Exec (
		const MIUNICODE *text, 		//!< Text to search
		UINT32 *pMatchStart = 0,	//!< Offset into text of start of matching text (pass NULL if don't care)
		UINT32 *pMatchEnd = 0,		//!< Offset into text of end of matching text (pass NULL if don't care)
		UINT32 textlen = ~0,			//!< Default is to use ucstrlen(text)
		int flags = 0
		) const;

#if 0
	#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Function to dump DFA to a text file for debugging
	void Write (
		FILE *out
		);
	#endif //!< GENERATING_DOXYGEN_OUTPUT
#endif

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT
	UINT32 m_flags;

	URESYMTAB *m_syms;
	MIUNICODE m_nsyms;

	UREDSTATE *m_states;
	MIUNICODE m_nstates;

	URETRANS *m_trans;
	MIUNICODE m_ntrans;

	friend struct UREBUFFER;
	#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef	CLASSLIBEXPORT



#endif
