/**
 * \file mi32/zview.h
 * \brief ZVIEW class definitions.
 *
 * \if NODOC
 * $Id: zview.h_v 1.5 2012/04/04 20:47:44 mju Exp $
 *
 * $Log: zview.h_v $
 * Revision 1.5  2012/04/04 20:47:44  mju
 * Add separate methods for left/right point shift.
 *
 * Revision 1.4  2012/03/30 21:13:51  mju
 * Make calcDisplacedPoint... inline.
 *
 * Revision 1.3  2012/03/30 17:28:45  mju
 * Use stereoshift.
 *
 * Revision 1.2  2012/01/19 18:41:42  mju
 * Keep surface data as memory array and add setup from pipeline stage.
 *
 * Revision 1.1  2011/12/01 22:21:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_ZVIEW_H
#define	INC_MI32_ZVIEW_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_STEREOSHIFT_H
#include <mi32/stereoshift.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class POLYLINE;
class SUBREGION2D;
class TRANS2D_MAPGEN;
namespace RVC {
	class OBJITEM;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}
#endif

//=====================================================================================================================
//! Manage z/elevation/surface and view/canvas interaction.
class ZVIEW {
	public:

		//! Constructor.
		ZVIEW ();

		//! Destructor.
		~ZVIEW ();

		//! Calculate dual-view displaced points from non-displaced for overhead stereo view using Z value specified.
		void CalcDisplacedPointDual (
			const DPOINT3D& ptBase,								//!< Base point in "device" coordinates
			DPOINT2D& ptLeft,										//!< Left displaced point in device coordinates returned
			DPOINT2D& ptRight										//!< Right displaced point in device coordinates returned
			) const { m_StereoShift.CalcPointDual(ptBase,ptLeft,ptRight); }

		//! Calculate dual-view displaced points from non-displaced for overhead stereo view using Z value from surface.
		void CalcDisplacedPointDualOnSurface (
			DPOINT3D& ptBase,										//!< Base point in "device" coordinates, Z value will be updated
			DPOINT2D& ptLeft,										//!< Left displaced point in device coordinates returned
			DPOINT2D& ptRight										//!< Right displaced point in device coordinates returned
			) const;

		//! Calculate left view displaced point from non-displaced for overhead stereo view using Z value specified.
		void CalcDisplacedPointLeft (
			const DPOINT3D& ptBase,								//!< Base point in "device" coordinates
			DPOINT2D& ptDisplaced								//!< Displaced point in device coordinates returned
			) const { m_StereoShift.CalcPointLeft(ptBase,ptDisplaced); }

		//! Calculate right view displaced point from non-displaced for overhead stereo view using Z value specified.
		void CalcDisplacedPointRight (
			const DPOINT3D& ptBase,								//!< Base point in "device" coordinates
			DPOINT2D& ptDisplaced								//!< Displaced point in device coordinates returned
			) const { m_StereoShift.CalcPointRight(ptBase,ptDisplaced); }

		//! Calculate single-view displaced point from non-displaced for overhead stereo view using Z value from surface.
		//! Should not be used if both views are displaced.
		void CalcDisplacedPointSingleOnSurface (
			DPOINT3D& ptBase,										//!< Point in "device" coordinates, Z value will be updated
			DPOINT2D& ptDisplaced								//!< Displaced point in device coordinates returned
			) const;

		//! Calculate dual-view displaced polylines from non-displaced for overhead stereo view, optionally draping on surface.
		//! Should only be used if both views are displaced.
		void CalcDisplacedPolylineDual (
			const POLYLINE& plBase,								//!< Polyline in "device" coordinates
			POLYLINE& plLeft,										//!< Left displaced polyline returned
			POLYLINE& plRight,									//!< Right displaced polyline returned
			bool UseSurfaceZ										//!< Use Z value from surface
			) const;

		//! Calculate single-view displaced polyline from non-displaced for overhead stereo view, optionally draping on surface.
		//! Should not be used if both views are displaced.
		void CalcDisplacedPolylineSingle (
			const POLYLINE& plBase,								//!< Polyline in "device" coordinates
			POLYLINE& plDisplaced,								//!< Displaced polyline returned
			bool UseSurfaceZ										//!< Use Z value from surface
			) const;

		//! Calculate dual-view displaced subregions from non-displaced for overhead stereo view, optionally draping on surface.
		//! Should only be used if both views are displaced.
		void CalcDisplacedSubRegionDual (
			const SUBREGION2D& srBase,							//!< Subregion in device coordinates
			SUBREGION2D& srLeft,									//!< Left displaced subregion returned
			SUBREGION2D& srRight,								//!< Right displaced subregion returned
			bool UseSurfaceZ										//!< Use Z value from surface
			) const;

		//! Calculate single-view displaced subregion from non-displaced for overhead stereo view, optionally draping on surface.
		//! Should not be used if both views are displaced.
		void CalcDisplacedSubRegionSingle (
			const SUBREGION2D& srBase,							//!< Subregion in device coordinates
			SUBREGION2D& srDisplaced,							//!< Displaced subregion returned
			bool UseSurfaceZ										//!< Use Z value from surface
			) const;

		//! Calculate Z value for single point.
		//! @return true if has Z value, false if outside defined surface.
		bool CalcPointZ (
			DPOINT3D& ptDevice									//!< Point in "device" coordinates, Z value will be updated
			) const;

		//! Close and release currently open surface objects.
		void CloseSurface ();

		//! Get stereo shift values.
		const STEREOSHIFT& GetStereoShift () const
			{ return (m_StereoShift); }

		//! Determine if has setup stereo shift.
		bool HasStereoShift () const
			{ return (m_StereoShift.IsValid()); }

		//! Determine if has setup terrain surface.
		bool HasSurface () const
			{ return (!m_SurfaceData.IsEmpty()); }

		//! Reset stereo displacement.
		void InvalidateStereoShift ()
			{ m_StereoShift.Invalidate(); }

		//! Set stereo shift values.
		void SetStereoShift (
			const STEREOSHIFT& StereoShift						//!< Values to set
			);

		//! Setup with surface object and transformation.
		//! Does not alter stereo displacement values.
		ERRVALUE SetupSurface (
			const RVC::OBJITEM& ObjItemSurface,					//!< Location of surface object
			const TRANS2D_MAPGEN& TransSurfaceToDevice,		//!< Translation from surface object to device coordinates
			const LRECT2D& DeviceRect								//!< Device rectangle
			);

		//! Setup with surface object and transformation.
		//! Does not alter stereo displacement values.
		ERRVALUE SetupSurface (
			SPATMOD::IMAGE::STAGE& StageSurface,				//!< Pipeline stage for reading surface
			const TRANS2D_MAPGEN& TransSurfaceToDevice,		//!< Translation from surface object to device coordinates
			const LRECT2D& DeviceRect								//!< Device rectangle
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		double m_SurfaceDensifyIncrement;				// Increment in pixels for densifying on surface
		STEREOSHIFT m_StereoShift;
		LPOINT2D m_Origin;
		LPOINT2D m_Size;
		SIMPLE_ARRAY<float> m_SurfaceData;

		bool GetSurfaceZ (INT32 x, INT32 y, double& z) const;

		ZVIEW (const ZVIEW&);
		ZVIEW& operator= (const ZVIEW&);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_ZVIEW_H
