/**
 * \file rvc/dbtableproperties.h 
 * \brief interface for the RVC::DBTABLE_PROPERTIES class.
 *
 * \if NODOC
 * $Id: dbtableproperties.h_v 1.1 2010/08/16 17:03:20 mju Exp $
 *
 * $Log: dbtableproperties.h_v $
 * Revision 1.1  2010/08/16 17:03:20  mju
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBTABLEPROPERTIES_H
#define INC_RVC_DBTABLEPROPERTIES_H

#ifndef INC_RVC_DBTABLE_H
	#include <rvc/dbtable.h>
#endif

namespace RVC {

//! Table to contain well-known properties, typically associated with main spatial object
class DBTABLE_PROPERTIES : public RVC::DBTABLE {
	public:

		//! Property ID.
		enum ID {
			ID_Invalid =			0,
			// Numeric IDs
			ID_Numeric =			0x00000000,
			ID_MinX =				ID_Numeric | 1,
			ID_MaxX =				ID_Numeric | 2,
			ID_MinY =				ID_Numeric | 3,
			ID_MaxY =				ID_Numeric | 4,
			ID_MinZ =				ID_Numeric | 5,
			ID_MaxZ =				ID_Numeric | 6,
			ID_PointDensity =		ID_Numeric | 7,	//!< Point density, count per square unit
			// String IDs
			ID_String =				0x10000000,
			// Masks
			ID_MASK_Type =			0xF0000000
			};

		//! Parameters to make properties table.
		class MAKEPARM : public RVC::DBTABLE::MAKEPARMS {
			public:
				//! Constructor.
				MAKEPARM ();
			};

		//! Property record.
		class RECORD : public RVC::DBTABLE::RECORD {
			public:

				//! Get ID from record.
				ID GetID () const
					{ return (static_cast<ID>(*static_cast<const UINT32*>(GetFieldPtr(FIELDNUM_ID)))); }

				//! Get numeric value.
				double GetNumeric () const
					{ return (*static_cast<const double*>(GetFieldPtr(FIELDNUM_Numeric))); }

				//! Get string value.
				void GetString (
					MISTRING& str
					) const { GetValue(FIELDNUM_String,str); }

				//! Set numeric property.
				void SetNumeric (
					ID id,
					double value
					) {
					*static_cast<INT32*>(GetFieldPtr(FIELDNUM_ID)) = id;
					SetValue(FIELDNUM_Numeric,value);
					}

				//! Set string property.
				void SetString (
					ID id,
					const MISTRING& str
					) {
					*static_cast<INT32*>(GetFieldPtr(FIELDNUM_ID)) = id;
					SetValue(FIELDNUM_String,str);
					}
			};

		//! Default constructor
		DBTABLE_PROPERTIES ();

		//! Copy constructor
		DBTABLE_PROPERTIES (
			const RVC::DBTABLE_PROPERTIES& rhs
			);

		//! Destructor
		virtual ~DBTABLE_PROPERTIES ();

		//! Assignment
		RVC::DBTABLE_PROPERTIES& operator= (
			const RVC::DBTABLE_PROPERTIES& rhs
			);

		//! Read numeric property value.
		//! @return TRUE if found, FALSE if not, < 0 if error.
		int ReadValue (
			ID id,						//!< Property ID
			double& value				//!< Value returned, will be unchanged if not found
			) const;
			
		//! Read string property value.
		//! @return TRUE if found, FALSE if not, < 0 if error.
		int ReadValue (
			ID id,						//!< Property ID
			MISTRING& value			//!< Value returned, will be unchanged if not found
			) const;

		//! Write numeric property value.
		ERRVALUE WriteValue (
			ID id,						//!< Property ID
			double& value				//!< Value to write
			);
			
		//! Write string property value.
		ERRVALUE WriteValue (
			ID id,						//!< Property ID
			const MISTRING& value	//!< Value to write
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum FIELDNUM {
			FIELDNUM_ID =			0,
			FIELDNUM_Numeric =	1,
			FIELDNUM_String =		2,
			FIELDNUM_COUNT
			};

		mutable RECORD m_RecordTemp;
		mutable SIMPLE_ARRAY<INT32> m_RecNums;

		int ReadRecord (ID id) const;
		ERRVALUE WriteRecord (ID id);
	
		virtual bool v_IsValidTable (const DBTABLE& table);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};


}		// End of RVC namespace

#endif 	// INC_RVC_DBTABLEPROPERTIES_H
