/**
 * \file rvc/hypersensor.h 
 * \brief interface for the RVC::HYPERSENSOR class.
 *
 * \if NODOC
 * $Id: hypersensor.h_v 1.2 2007/09/06 17:18:49 scowan Exp $
 *
 * $Log: hypersensor.h_v $
 * Revision 1.2  2007/09/06 17:18:49  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.1  2006/03/03 21:11:42  scowan
 * Initial revision
 *
 * Revision 1.1  2006/03/03 18:49:51  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::HYPERSENSOR rvc/hypersensor.h
//!	\brief Manages RVC::OBJTYPE_HyperspectralSensorInfo RVC objects.
//!
//!	The interface currently wraps the MfSensorInfo functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_HyperspectralSensorInfo
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_HyperspectralSensorInfo
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Hyperspectral
//!
//!	Read(), Make() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods


#ifndef INC_RVC_HYPERSENSOR_H
#define INC_RVC_HYPERSENSOR_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class DATETIME;
#endif // GENERATING_DOXYGEN_OUTPUT


namespace RVC {

class HYPERSENSOR : public RVC::SIMPLE {
	public:
		enum SENSOR {		//! Some initial sensor definitions, this field is just for information only.
			SENSOR_Unknown = 0,
			SENSOR_AVIRIS = 1,
			SENSOR_PROBE1 = 2,
			SENSOR_CASI = 3,
			SENSOR_HYMAP = 4,
			SENSOR_DIAS = 5,
			SENSOR_ORBVIEW4 = 6
			};
			
		struct BANDINFO {
			double m_wavelength;				//!< Band's center wavelength, mkm 
			double m_bandwidth; 				//!< Band's bandwidth, mkm 
			double m_scale; 	  				//!< Simple calibration to apparent reflectance 
			double m_offset;	  				//!< Reflectance = (raw - offset) / scale 
			};			

		HYPERSENSOR (
			);

		HYPERSENSOR (
			const RVC::HYPERSENSOR& rhs
			);

		virtual ~HYPERSENSOR (
			);

		RVC::HYPERSENSOR& operator= (
			const RVC::HYPERSENSOR& rhs
			);

		double GetAltitude (
			) const { return (m_altitude); }

		double GetAtmosphereThickness (
			) const { return (m_athmthickness); }
			
		const DOUBLE_ARRAY<BANDINFO>& GetBandInfo (
			) const { return (m_BandArray); }
		
		void GetImageDate (
			DATETIME& ImageDate
			) const;

		double GetInitWavelength (
			) const { return (m_initwavelength); }

		double GetLastWavelength (
			) const { return (m_lastwavelength); }

		double GetMaxBandwidth (
			) const { return (m_maxbandwidth); }

		double GetMinBandwidth (
			) const { return (m_minbandwidth); }
			
		INT32 GetNumBands (
			) const { return (m_NumBands); }

		SENSOR GetSensorType (
			) const { return (m_sensor); }

		double GetSunAngle (
			) const { return (m_sunangle); }
			
		void SetAltitude (
			double Altitude
			) { m_altitude = Altitude; }

		void SetAtmosphereThickness (
			double AtmosphereThickness			//!< Equivalent athmospheric thickness through which the observation was obtained 
			) { m_athmthickness = AtmosphereThickness; }
		
		void SetBandInfo (
			const DOUBLE_ARRAY<BANDINFO>& BandInfo
			) { m_BandArray = BandInfo; m_NumBands = BandInfo.GetNumItems(); }
		
		void SetImageDate (
			const DATETIME& ImageDate
			);

		void SetInitWavelength (
			double Wavelength
			) { m_initwavelength = Wavelength; }

		void SetLastWavelength (
			double Wavelength
			) { m_lastwavelength = Wavelength; }

		void SetMaxBandwidth (
			double Bandwidth
			) { m_maxbandwidth = Bandwidth; }

		void SetMinBandwidth (
			double Bandwidth
			) { m_minbandwidth = Bandwidth; }

		void SetSensorType (
			SENSOR sensor
			) { m_sensor = sensor; }

		void SetSunAngle (
			double SunAngle							//!< Sun angle at the moment of the survey, deg 
			) { m_sunangle = SunAngle; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RVCDATETIME	m_imagedate;				//!< Time when image was taken 
		SENSOR m_sensor;							//!< SENSOR_ constant 
		INT32 m_NumBands;							//!< Number of bands 
		double m_altitude;			 			//!< Sensor altitude at the moment of the survey, km 
		double m_sunangle;			 			//!< Sun angle at the moment of the survey, deg 
		double m_athmthickness;					//!< Equivalent athmospheric thickness through which the observation was obtained 
		double m_initwavelength;				//!< Initial Wavelength in mkm 
		double m_lastwavelength;				//!< Last Wavelength in mkm 
		double m_minbandwidth;			 		//!< Minimum Bandwidth in mkm 
		double m_maxbandwidth;			 		//!< Maximum Bandwidth in mkm 
		DOUBLE_ARRAY<BANDINFO> m_BandArray;
		
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_HYPERSENSOR_H
