/**
 * \file rvc/simple.h 
 * \brief interface for the RVCSIMPLE class.
 *
 * \if NODOC
 * $Id: simple.h_v 1.28 2007/09/06 17:19:01 scowan Exp $
 *
 * $Log: simple.h_v $
 * Revision 1.28  2007/09/06 17:19:01  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.27  2006/06/27 15:05:05  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.26  2005/12/19 18:05:04  dwilliss
 * modified default credentials to account for prior default parameters
 *
 * Revision 1.25  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.24  2005/12/09 17:29:55  scowan
 * Updated to use credentials.
 *
 * Revision 1.23  2005/07/19 17:49:12  scowan
 * Update to use current simple methods.
 *
 * Revision 1.22  2005/05/05 19:25:00  dfriberg
 * Modified comments for Make (RVC::OBJITEM, const char*)
 *
 * Revision 1.21  2005/03/31 16:57:27  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.20  2005/03/07 18:30:35  scowan
 * Removed unused methods.
 *
 * Revision 1.19  2005/01/19 23:37:49  scowan
 * Undo must override virtual method.
 *
 * Revision 1.18  2005/01/19 21:19:55  scowan
 * Added support for descriptor.
 *
 * Revision 1.17  2004/02/06 22:36:33  scowan
 * Added methods to get the default name and desc.
 *
 * Revision 1.16  2003/10/07 17:30:40  scowan
 * Object virtual method changes.
 *
 * Revision 1.15  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.14  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.13  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.12  2003/05/13 14:06:53  scowan
 * Added override to open object.
 *
 * Revision 1.11  2002/12/06 15:10:00  scowan
 * Fixed for genator docs.
 *
 * Revision 1.10  2002/04/05 22:34:17  scowan
 * Cleaned up.
 *
 * Revision 1.9  2002/01/09 20:13:12  scowan
 * Added source parameter for make method.
 *
 * Revision 1.8  2001/06/15 16:17:41  scowan
 * Added simple read method for current object.
 *
 * Revision 1.7  2001/06/14 16:14:23  scowan
 * Changed source type..
 *
 * Revision 1.6  2001/06/13 21:38:43  scowan
 * Added to RVC namespace.
 *
 * Revision 1.5  2001/06/13 14:54:44  scowan
 * Added new make method.
 *
 * Revision 1.4  2000/12/01 14:58:34  scowan
 * MAC errors.
 *
 * Revision 1.3  2000/12/01 14:54:00  scowan
 * Fixed MAC errors.
 *
 * Revision 1.2  2000/10/19 21:14:51  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 15:26:55  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::SIMPLE rvc/simple.h
//!	\brief Interface for RVC objects that support a Read Once, Write Once
//!	paradigm.  Derived classes do not open / close the rvc object.
//!
//!	This class inherits from RVC::OBJECT
//!	
//!	RVC Object Types that this class supports: Derived class
//!
//!	Default RVC Object Type for this class: Derived class
//!
//!	Valid parents for this class: Derived class

#ifndef INC_RVC_SIMPLE_H
#define INC_RVC_SIMPLE_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

namespace RVC {
#ifndef GENERATING_DOXYGEN_OUTPUT
class OBJMAKEPARMS;		// Forward Declarations
class OBJOPENPARMS;
#endif // GENERATING_DOXYGEN_OUTPUT

class SIMPLE : public RVC::OBJECT {			// Specify namespace in inheritance so that other "OBJECT's" do not get used as the parent
	public:

		//! Default Constructor
		SIMPLE (
			);

		//! Copy constructor
		SIMPLE (
			const RVC::SIMPLE& rhs
			);

		//! Destructor, pure virtual
		virtual ~SIMPLE (							//! Destructor, pure virtual
			) = 0;

		//! Assignment
		RVC::SIMPLE& operator= (						//! Assignment operator
			const RVC::SIMPLE& rhs
			);

		//! Obtain default name and description information from derived class
		ERRVALUE GetDftNameDesc (			
			const RVC::OBJITEM& ParentObjItem,		//!< Parent rvc object item
			RVC::DESCRIPTOR& Descriptor				//!< Default generated descriptor RETURNED
			);

		//! Obtain default name and description information from derived class
		ERRVALUE GetDftNameDesc (			
			const RVC::OBJECT& ParentObject,			//!< Parent rvc object
			RVC::DESCRIPTOR& Descriptor				//!< Default generated descriptor RETURNED
			);

		//! Create new rvc object of derived class type using default name and desc.
		//! This method will attach this instance to the rvc object after writing the data.  This instance file handle will be closed
		ERRVALUE Make (
			const RVC::OBJECT& parent,					//!< Parent rvc object to create derived class under
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
			const CREDENTIALS* Credentials 			//!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,							//!< Who created this rvc object, 0 to use result from GetAppName()
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Create new rvc object of derived class type with given name and desc.
		//! This method will attach this instance to the rvc object after writing the data.  This instance file handle will be closed
		ERRVALUE Make (
			const RVC::OBJECT& parent,					//!< Parent rvc object to create derived class under
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create new rvc object of derived class type with given OBJITEM.  
		//! This method will attach this instance to the parent of the ObjItem after writing the data.  This instance file handle will be closed
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,						//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
			const CREDENTIALS* Credentials 			//!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,							//!< Who created this rvc object, 0 to use result from GetAppName()
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read data from rvc object, data stored in derived class
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was open before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from named rvc object, data stored in derived class
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was open before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const FILEPATH& filepath,					//!< Path to RVC file
			const MISTRING& ObjectPath,					//!< Path to requested rvc object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from rvc object, data stored in derived class.
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was open in this instance before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const RVC::OBJECT& object,					//!< RVC Object to read data from
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from found rvc object, data stored in derived class.
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was in this instance open before the call, this method still closes the file.  This is by design.
		//! @deprecated Use the method that takes a DESCRIPTOR
		DEPRECATED ERRVALUE Read (
			const RVC::OBJECT& parent,					//!< Parent rvc object
			const RVC::OBJECTNAME& name,					//!< Name of rvc object to locate and read from
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from found rvc object, data stored in derived class.
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was in this instance open before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const RVC::OBJECT& parent,					//!< Parent rvc object
			const RVC::DESCRIPTOR& Descriptor,		//!< Name of rvc object to locate and read from
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from defined rvc object, data stored in derived class.
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was in this instance open before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const RVC::OBJITEM& ObjItem,			//!< This contains the file and rvc object location information
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read data from found rvc object, data stored in derived class.
		//!
		//! This method will open the file, if not open already, and after reading the data, close its instance of the file.
		//! If the file was in this instance open before the call, this method still closes the file.  This is by design.
		ERRVALUE Read (
			const RVC::OBJECT& source,			//!< Source rvc object used if 'filename' is a relative path
			const MIUNICODE *filename,			//!< Full or relative path to the rvc file to open
			const MISTRING& ObjectPath,			//!< Full name of rvc object obtained from GetObjectPath()
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Overwrite existing derived class type with current information.
		//!
		//! This method will open the file, if not open already, and after writing the data, close its instance of the file.
		//! If the file was open in this instance before the call, this method still closes the file.  This is by design.
		ERRVALUE Write (
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

	private:
	
		//! Obtain default name and description information from derived class
		virtual ERRVALUE v_GetDftNameDesc (
			const OBJECT& parent,						//!< Parent rvc object to create derived class under
			DESCRIPTOR& Descriptor
			) = 0;

		virtual ERRVALUE v_MakeData (					//! Make specific data method for derived classes
			RVC::OBJMAKEPARMS& ObjMakeParms
			) = 0;
			
		virtual ERRVALUE v_ReadData (					//! Read specific data method for derived classes
			RVC::OBJOPENPARMS& ObjOpenParms
			) = 0;

		virtual ERRVALUE v_WriteData (				//! Write specific data method for derived classes
			RVC::OBJOPENPARMS& ObjOpenParms
			) = 0;
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif 		//!< INC_RVC_SIMPLE_H
