/**
 * \file spatmod/imageautoregister.h
 * \brief SPATMOD::IMAGE::AUTOREGISTER definitions
 *
 * \if NODOC
 * $Id: imageautoregister.h_v 1.9 2011/10/11 13:57:24 mju Exp $
 *
 * $Log: imageautoregister.h_v $
 * Revision 1.9  2011/10/11 13:57:24  mju
 * Remove unneeded hdr.
 *
 * Revision 1.8  2011/02/23 18:09:39  mju
 * Remove max gcps setting.
 *
 * Revision 1.7  2011/01/17 17:55:29  mju
 * Add settings for predefined and using existing gcps.
 *
 * Revision 1.5  2011/01/04 21:26:38  mju
 * Remove maxRunTime, combine settings.
 *
 * Revision 1.1  2010/12/13 17:51:56  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEAUTOREGISTER_H
#define	INC_SPATMOD_IMAGEAUTOREGISTER_H

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

#ifndef	INC_SPATMOD_GEOREFERENCE_H
#include <spatmod/georeference.h>
#endif

// Forward declarations
#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SPATMOD {
	class GEOREFERENCE;
	namespace IMAGE {
		class STAGE;
		}
	}
namespace RVC {
	class OBJITEMLIST;
	}
#endif


namespace SPATMOD {
namespace IMAGE {

//! Perform automatic registration of input image to reference.
class AUTOREGISTER {
	public:

		//! Settings used for Run.
		class SETTINGS {
			public:

				//! Constructor, reads setting from Ini file.
				SETTINGS ();

				//! Copy constructor.
				SETTINGS (const SETTINGS& rhs);

				//! Destructor.
				~SETTINGS ();

				//! Assignment.
				SETTINGS& operator= (const SETTINGS& rhs);

				//! Get whether to adjust existing GCPs if used.
				bool GetAdjustExistingGCPs () const
					{ return (m_AdjustExistingGCPs); }

				//! Get whether to automatically generate GCPs.
				bool GetAutoGenerateGCPs () const
					{ return (m_AutoGenerateGCPs); }

				//! Get autocorrelation window size in input image cells.
				UINT16 GetCorrPatchSize () const
					{ return (m_CorrPatchSize); }

				//! Get initial accuracy in input image cells.
				double GetInitialAccuracy () const
					{ return (m_InitialAccuracy); }

				//! Get GCP spacing in input image cells.
				double GetGCPSpacing () const
					{ return (m_GCPSpacing); }

				//! Get maximum individual GCP residual in input image cells.
				double GetMaxGCPResidual () const
					{ return (m_MaxGCPResidual); }

				//! Set maximum model for automatic determination.
				GEOREFERENCE::MODEL GetMaxModel () const
					{ return (m_MaxModel); }

				//! Get list of objects containing predefined GCPs.
				const RVC::OBJITEMLIST* GetPredefinedGCPs () const
					{ return (m_pObjItemListPredefinedGCPs); }

				//! Get resample method.
				RESAMPLEMETHOD GetResampleMethod () const
					{ return (m_ResampleMethod); }

				//! Get whether to use existing GCPs.
				bool GetUseExistingGCPs () const
					{ return (m_UseExistingGCPs); }

				void IniWrite () const;

				void ResetDefaults ();

				//! Set to automatically generate GCPs.
				void SetAutoGenerateGCPs (
					bool AutoGenerateGCPs
					) { m_AutoGenerateGCPs = AutoGenerateGCPs; }

				//! Set autocorrelation window size in input image cells.
				void SetCorrPatchSize (UINT16 CorrPatchSize)
					{ m_CorrPatchSize = CorrPatchSize; }

				//! Set GCP spacing in input image cells.
				void SetGCPSpacing (double GCPSpacing)
					{ m_GCPSpacing = GCPSpacing; }

				//! Set initial accuracy in input image cells.
				void SetInitialAccuracy (double InitialAccuracy)
					{ m_InitialAccuracy = InitialAccuracy; }

				//! Set maximum individual GCP residual in input image cells.
				void SetMaxGCPResidual (double MaxGCPResidual)
					{ m_MaxGCPResidual = MaxGCPResidual; }

				//! Set maximum model for automatic determination.
				void SetMaxModel (GEOREFERENCE::MODEL Model)
					{ m_MaxModel = Model; }

				//! Set predefined GCPs to use.
				void SetPredefinedGCPs (
					const RVC::OBJITEMLIST *pObjItemListPredefinedGCPs		//!< Object location, NULL to not use
					);

				//! Set resample method.
				void SetResampleMethod (RESAMPLEMETHOD ResampleMethod)
					{ m_ResampleMethod = ResampleMethod; }

				//! Set to use existing GCPs.
				void SetUseExistingGCPs (
					bool UseExisting,
					bool AdjustExisting
					) { m_UseExistingGCPs = UseExisting; m_AdjustExistingGCPs = AdjustExisting; }

			private:
				#ifndef	GENERATING_DOXYGEN_OUTPUT
				double m_InitialAccuracy;
				double m_GCPSpacing;
				double m_MaxGCPResidual;
				UINT16 m_CorrPatchSize;
				GEOREFERENCE::MODEL m_MaxModel;
				RESAMPLEMETHOD m_ResampleMethod;
				RVC::OBJITEMLIST *m_pObjItemListPredefinedGCPs;
				bool m_AutoGenerateGCPs;
				bool m_UseExistingGCPs;
				bool m_AdjustExistingGCPs;

				void Copy (const SETTINGS& rhs);
				#endif	// GENERATING_DOXYGEN_OUTPUT
			};

		AUTOREGISTER ();
		~AUTOREGISTER ();

		//! Get georeference resulting from autoregistration.
		//! Use to access control points and model.
		const GEOREFERENCE* GetGeorefResult () const;

		//! Get RMS residual from last run.
		double GetRMSResidual () const;

		//! Run autoregistration.
		//! Will only return error if cannot determine any reasonable georeference for input.
		int Run (
			IMAGE::STAGE& StageInput,				//!< Input image, must have initial georeference
			IMAGE::STAGE& StageRef,					//!< Reference image, must have georeference
			const SETTINGS& Settings				//!< Run settings
			);

	private:
		#ifndef	GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		AUTOREGISTER (const AUTOREGISTER&);
		AUTOREGISTER& operator= (const AUTOREGISTER&);
		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEAUTOREGISTER_H
