/**
 * \file spatmod/imagefilterscaleoffset.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_SCALEOFFSET definition.
 *
 * \if NODOC
 *
 * $Id: imagefilterscaleoffset.h_v 1.3 2009/12/01 16:19:05 mju Exp $
 *
 * $Log: imagefilterscaleoffset.h_v $
 * Revision 1.3  2009/12/01 16:19:05  mju
 * Add setDoFloor.
 *
 * Revision 1.2  2008/08/28 13:58:21  mju
 * Fix doxygen file ref.
 *
 * Revision 1.1  2006/01/12 21:22:18  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSCALEOFFSET_H
#define	INC_SPATMOD_IMAGEFILTERSCALEOFFSET_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

class MIMATRIX;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to apply scale and offset to samples in image.
//! If applied to integer data resulting values will be rounded.
class FILTER_SCALEOFFSET : public FILTER {
	public:

		//! Constructor.
		FILTER_SCALEOFFSET (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership						//!< Filter takes ownership of previous stage and will delete when needed
			);

		//! Destructor.
		virtual ~FILTER_SCALEOFFSET ();

		//! Get scale for sample.
		//! If sample index is out of range then 0 will be returned.
		double GetScale (
			UINT16 SampleNum							//!< Sample index
			) const;

		//! Get scales as MIMATRIX.
		void GetScales (
			MIMATRIX& Scales							//!< Scale matrix returned
			) const;

		//! Get offset for sample.
		//! If sample index is out of range then 0 will be returned.
		double GetOffset (
			UINT16 SampleNum							//!< Sample index
			) const;

		//! Get offsets as MIMATRIX.
		void GetOffsets (
			MIMATRIX& Offsets							//!< Offset matrix returned
			) const;

		//! Set to use "floor" when processing integer data (default is to round).
		void SetDoFloor (
			bool DoFloor = true
			) { m_DoFloor = DoFloor; }

		//! Set constant for sample.
		ERRVALUE SetScaleOffset (
			UINT16 SampleNum,							//!< Sample index
			double scale,								//!< Scale to set
			double offset								//!< Offset value to add after applying scale
			);

		//! Set constant scales for all samples.
		//! Matrix must be vector of with size corresponding to number of samples.
		ERRVALUE SetScaleOffsets (
			const MIMATRIX& Scales,					//!< Scales as vector matrix
			const MIMATRIX& Offsets					//!< Offsets as vector matrix
			);

		//! Set constant scales for all samples.
		void SetScaleOffsets (
			const double *Scales,					//!< Scale array, must have size based on number of samples.
			const double *Offsets					//!< Offset array, must have size based on number of samples.
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DOUBLE_ARRAY<double> m_Scales;
		DOUBLE_ARRAY<double> m_Offsets;
		bool m_CheckRange;
		bool m_DoFloor;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_SCALEOFFSET (const FILTER_SCALEOFFSET&);
		FILTER_SCALEOFFSET& operator= (const FILTER_SCALEOFFSET&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSCALEOFFSET_H

