/**
 * \file <spatmod/imagefilterstereo.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_STEREO definition.
 *
 * \if NODOC
 * $Id: imagefilterstereo.h_v 1.4 2011/10/11 14:34:51 mju Exp $
 *
 * $Log: imagefilterstereo.h_v $
 * Revision 1.4  2011/10/11 14:34:51  mju
 * Use trans2dmapgen hdr.
 *
 * Revision 1.3  2011/09/29 12:25:13  mju
 * Use stereodevice class.
 *
 * Revision 1.2  2008/12/16 16:01:53  mju
 * Have separate members for resampling method drape vs terrain.
 *
 * Revision 1.1  2007/04/27 17:47:59  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSTEREO_H
#define	INC_SPATMOD_IMAGEFILTERSTEREO_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

#ifndef	INC_MI32_TRANS2MAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_STEREODEVICE_H
#include <mi32/stereodevice.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to generate stereo image.
//! This filter creates either a left or right epipolar image and then combines it with the "normal" image using the
//! default or specified stereo combination methods (anaglyph or interlacing).  This filter will avoid processing of
//! any pixels or samples that are ultimately excluded from the rendering based on the stereo combination method.
//! Further filtering of resulting image is not recommended as most filters will alter the image in ways unsuitable
//! for stereo viewing.
//!
//! The drape stage must have a pixel type of RGB for the analyph and Sharp stereo methods.
//! All samples from the drape stage must have the same data type.
//! The terrain stage must have a single component of any numeric data type.
//!
//! The value for DisplacementScale should be chosen such that the maximum actual displacement is reasonably small,
//! typically no more than 50 pixels, both in order to maintain visual stereo fusion, as well as for processing efficiency.
//! This value effectively controls the "vertical exaggeration" and relative "depth" of the rendering.
//! The value for DisplacementZeroValue must be within the range of the overall terrain value and typically will be
//! near the maximum terrain value for a particular rendering.  This may require implementation of user controls
//! for adjustment, or a preprocessing pass may be made to estimate this value based on the portion of the terrain used
//! in the rendering.  Rendered pixels on the terrain "above" the "zero displacement" will appear "closer" to the viewer
//! than the rendering plane (monitor screen).  Pixels on the terrain far "below" the "zero displacement" level may appear
//! with too much separation.
//!
//! The coordinate transformation between resulting image and input stage is computed using georeference as
//! specified in constructor, or georeference from specified reference image, along with the georeference
//! from the input stage.  An optional transformation accuracy may be specified via SetAccuracy() to allow use
//! of efficient affine transformation if appropriate.
//! Note that source image will not be processed "sequentially", and the sections specified
//! in PullBegin for the source will overlap.  It is recommended that this filter be placed early in the pipeline or
//! that a suitable caching filter be placed ahead of it if the input requires significant computation or sequential
//! iteration.
class FILTER_STEREO : public FILTER {
	public:

		//! Construct using specified dimensions and georeference.
		FILTER_STEREO (
			STAGE *pStageDrape,						//!< Pointer to stage to "drape" onto terrain
			bool TakeOwnershipDrape,				//!< Filter takes ownership of drape stage and will delete when needed
			STAGE *pStageTerrain,					//!< Pointer to stage for terrain elevation
			bool TakeOwnershipTerrain,				//!< Filter takes ownership of terrain stage and will delete when needed
			double DisplacementScale,				//!< Terrain displacement scale in horizontal cells per vertical unit (-)left, (+)right
			double DisplacementZeroValue,			//!< Terrain value having zero displacement
			const DIMENSIONS& Dimensions,			//!< Resulting image dimensions
			const GEOREFERENCE& Georeference,	//!< Georeference for resulting image
			RESAMPLEMETHOD ResampleMethod,		//!< Resampling method
			bool InterlaceStartRight = false		//!< First interlaced pixel (column or row) is Right if true, Left if false
			);

		//! Construct to match specified reference image.
		FILTER_STEREO (
			STAGE *pStageDrape,						//!< Pointer to stage to "drape" onto terrain
			bool TakeOwnershipDrape,				//!< Filter takes ownership of drape stage and will delete when needed
			STAGE *pStageTerrain,					//!< Pointer to stage for terrain elevation
			bool TakeOwnershipTerrain,				//!< Filter takes ownership of terrain stage and will delete when needed
			double DisplacementScale,				//!< Terrain displacement scale in horizontal cells per vertical unit (-)left, (+)right
			double DisplacementZeroValue,			//!< Terrain value having zero displacement
			const BASE& RefImage,					//!< Reference image to match
			RESAMPLEMETHOD ResampleMethod,		//!< Resampling method
			bool InterlaceStartRight = false		//!< First interlaced pixel (column or row) is Right if true, Left if false
			);

		// Destructor.
		virtual ~FILTER_STEREO ();

		//! Set transformation accuracy in source cells.
		//! If within the specified accuracy, a fast affine transformation will be used for computation.
		//! If used, must be done before Initialize().
		//! Default accuracy is .5 cells.
		void SetAccuracy (
			double accuracy							//!< Transformation accuracy in source cells.
			) { m_Accuracy = accuracy; }

		//! Set rendering method parameters.
		//! If not used then current global settings will be used.
		void SetStereoDevice (
			const STEREODEVICE& StereoDevice
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STEREODEVICE m_StereoDevice;
		RESAMPLEMETHOD m_ResampleMethodDrape;
		RESAMPLEMETHOD m_ResampleMethodTerrain;
		TRANS2D_MAPGEN m_TransToPrev[2];
		DRECT2D m_RectTotalPrev[2];
		double m_DisplacementScale;
		double m_DisplacementZeroValue;
		double m_DisplacementMax;
		double m_Accuracy;
		bool m_InterlaceStartRight;
		bool m_PullSinglePixels;

		ERRVALUE ComputePrevMaxSectionSize (
			const SECTION_ITERATOR& TgtIterator,
			int StageIdx,
			DRECT2D& PrevRectAll,
			UINT32& PrevMaxColumns,
			UINT32& PrevMaxRows
			) const;

		ERRVALUE DoPullBegin (
			const SECTION_ITERATOR& TgtIterator,
			int StageIdx
			);

		ERRVALUE GetTerrainDisplacement (
			DOUBLE_ARRAY<double>& TerrainShift,
			const DOUBLE_ARRAY<DPOINT2D>& TerrainCoord,
			BUFFER& TerrainBuffer,
			INT32 TerrainBaseRow,
			INT32 TerrainBaseColumn
			) const;

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_STEREO (const FILTER_STEREO&);
		FILTER_STEREO& operator= (const FILTER_STEREO&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSTEREO_H

