/**
 * \file imagesourceregion.h <spatmod/imagesourceregion.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_REGION definition.
 *
 * \if NODOC
 * $Id: imagesourceregion.h_v 1.4 2009/04/27 20:35:25 mju Exp $
 *
 * $Log: imagesourceregion.h_v $
 * Revision 1.4  2009/04/27 20:35:25  mju
 * Add pObjItemTempRaster.
 *
 * Revision 1.3  2008/11/17 17:37:13  mju
 * Remove transToCRS parm from ctor using refimage.
 * Remove ctor that does not take dimensions.
 *
 * Revision 1.2  2007/12/21 16:58:26  scowan
 * Added lrect 2d to store the extents of the region in image coordinates
 *
 * Revision 1.1  2007/12/20 23:25:39  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEREGION_H
#define	INC_SPATMOD_IMAGESOURCEREGION_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
namespace RVC {
	class OBJITEM;
	}
#endif


namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports the REGION2D as a mask.  The inside of the region is value '1' and the outside is value '0'.
//! If the REGION2D is empty, an error will result in the call to Initialize().  
class SOURCE_REGION : public SOURCE {
	public:

		//! Construct to match reference image.
		//! The size of the source is determined by 'RefImage'.
		//! The decision whether to actually use memory is done during the Initialize() step.
	 	SOURCE_REGION (
			const REGION2D& Region,					//!< Region to use
			const BASE &RefImage,					//!< Reference image
			double MemoryMaxUsage = 4.0			//!< Memory usage threshold (0 = none, < .5 = fraction of available, > .5 = MB)
			);
			
		//! Construct with specified dimensions and image/map transformation.
		//! The size of the source is determined by 'TotalRows' and 'TotalColumns'.
		//! The decision whether to actually use memory is done during the Initialize() step.
	 	SOURCE_REGION (
			const REGION2D& Region,					//!< Region to use
			const TRANS2D_MAPGEN& TransToCRS,	//!< Input section must contain transformation from image to a map CRS, not necessarily the one the 'region' is in.
															//!< Output section is ignored.
															//!< If 'Region' is already in 'RefImage' coordinates, pass an identity transformation.
			UINT32 TotalColumns, 					//!< Total columns
			UINT32 TotalRows, 						//!< Total rows
			double MemoryMaxUsage = 4.0			//!< Memory usage threshold (0 = none, < .5 = fraction of available, > .5 = MB)
			);

		//! Destructor.
		virtual ~SOURCE_REGION ();
		
		//! Obtain the extents of the region in image coordinates.  Only valid after call to Initialize().
		//! Can be use as a parameter to FILTER_CROP.
		//! @return Extents of the region in image coordinates.
		const LRECT2D& GetRegionExtents (
			) const { return (m_RegionImageExtents); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SOURCE *m_pCache;
		double m_MemoryMaxUsage;
		bool m_UseMemory;
		REGION2D m_Region;
		LRECT2D m_RegionImageExtents;
		RVC::OBJITEM *m_pObjItemTempRaster;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_REGION (const SOURCE_REGION&);
		SOURCE_REGION& operator= (const SOURCE_REGION&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEREGION_H

